package com.example.civilizedcity.common;

import com.example.civilizedcity.entity.SysOrg;
import com.example.civilizedcity.entity.TaAttach;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.entity.TaOrgIssue;
import com.example.civilizedcity.service.SysOrgService;
import com.example.civilizedcity.service.TaAttachService;
import com.example.civilizedcity.service.TaOrgIssueService;
import com.example.civilizedcity.vo.IssueExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.net.MalformedURLException;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Component
public class ExportIssueUtil {

    @Autowired
    private SysOrgService sysOrgService;

    @Autowired
    private TaAttachService taAttachService;

    @Autowired
    private TaOrgIssueService taOrgIssueService;

    public List<IssueExport> getExportList(List<TaIssue> list, Boolean withImage) throws MalformedURLException {
        List<IssueExport> exportList = new ArrayList<>();
        if (null != list && list.size() > 0) {
            List<SysOrg> orgList = sysOrgService.list();

            for (TaIssue taIssue : list) {
                IssueExport issueExport = new IssueExport();
                issueExport.setIssueId(taIssue.getIssueId());
                issueExport.setTypeName(taIssue.getTypeName());
                issueExport.setCreateDate(taIssue.getCreateDate());
                issueExport.setLocName(taIssue.getLocName());
                issueExport.setAddr(taIssue.getAddr());
                issueExport.setContent(taIssue.getContent());
                issueExport.setExpireDate(taIssue.getExpireDate());
                issueExport.setProcessNum(taIssue.getProcessNum());
                issueExport.setUserName(taIssue.getUserName());
                issueExport.setEndDate(taIssue.getEndDate());
//                issueExport.setSourceType(taIssue.getSourceType());
//                此处是PC端excel表展示
                String sourceType2=taIssue.getSourceType();
                if(sourceType2.equals(Constants.SOURCE_FEEDBACK)){
                    issueExport.setSourceType2("市民上报");
                }else if(sourceType2.equals(Constants.SOURCE_CHECK)){
                    issueExport.setSourceType2("实地测评");
                }else if(sourceType2.equals(Constants.SOURCE_INSPECTOR)){
                    issueExport.setSourceType2("日常督查");
                }

                // 转换机构
                if (StringUtils.isNotEmpty(taIssue.getOrgId())) {
                    Optional<SysOrg> sysOrg = orgList.stream().filter(x -> x.getOrgId().equals(taIssue.getOrgId())).findFirst();
                    if (sysOrg.isPresent()) {
                        issueExport.setOrgId(sysOrg.get().getName());
                    } else {
                        issueExport.setOrgId(null);
                    }
                }

                // 转换流程状态
                String processNode = taIssue.getProcessNode();
                if (processNode.equals(Constants.PROCESS_START)) {
                    issueExport.setProcessNode("待交办");
                } else if (processNode.equals(Constants.PROCESS_END)) {
                    issueExport.setProcessNode("已办结");
                } else if (StringUtils.isNotEmpty(taIssue.getExpireDate()) && LocalDateTime.now().isAfter(DateUtils.day2LocalDateime2(taIssue.getExpireDate()))) {
                    issueExport.setProcessNode("已逾期");
                } else {
                    issueExport.setProcessNode("已交办");
                }

                // 默认是带图的
                if (null == withImage || withImage) {
                    // 获取图片列表
                    List<TaAttach> attachList = taAttachService.getListBy(Constants.SOURCE_ISSUE, taIssue.getIssueId().toString());
                    if (null != attachList && attachList.size() > 0) {
                        List<TaAttach> images = attachList.stream().filter(x -> StringUtils.isEmpty(x.getFileType()) || x.getFileType().equals("image")).collect(Collectors.toList());

                        int max = 5;
                        for (int i = 0; i < max; i += 1) {
                            TaAttach taAttach = null;

                            try {
                                taAttach = images.get(i);
                            } catch (Exception e) {
                                //
                            }

                            if (null != taAttach) {

                                URL url = new URL(taAttach.getUrl());
                                switch (i) {
                                    case 0:
                                        issueExport.setUrl1(url);
                                        break;
                                    case 1:
                                        issueExport.setUrl2(url);
                                        break;
                                    case 2:
                                        issueExport.setUrl3(url);
                                        break;
                                    case 3:
                                        issueExport.setUrl4(url);
                                        break;
                                    case 4:
                                        issueExport.setUrl5(url);
                                        break;
                                    default:
                                        break;
                                }
                            }
                        }
                    }

                    //
                    TaOrgIssue taOrgIssue = taOrgIssueService.getByIssueAndOrg(taIssue.getIssueId(), taIssue.getOrgId());
                    if (null != taOrgIssue) {
                        List<TaAttach> orgAttachList = taAttachService.getListBy(Constants.SOURCE_ORG_ISSUE, taOrgIssue.getOrgIssueId().toString());
                        if (null != orgAttachList && orgAttachList.size() > 0) {
                            List<TaAttach> images = orgAttachList.stream().filter(x -> StringUtils.isEmpty(x.getFileType()) || x.getFileType().equals("image")).collect(Collectors.toList());

                            int max = 5;
                            for (int i = 0; i < max; i += 1) {
                                TaAttach taAttach = null;

                                try {
                                    taAttach = images.get(i);
                                } catch (Exception e) {
                                    //
                                }

                                if (null != taAttach) {
                                    URL url = new URL(taAttach.getUrl());
                                    switch (i) {
                                        case 0:
                                            issueExport.setUrl6(url);
                                            break;
                                        case 1:
                                            issueExport.setUrl7(url);
                                            break;
                                        case 2:
                                            issueExport.setUrl8(url);
                                            break;
                                        case 3:
                                            issueExport.setUrl9(url);
                                            break;
                                        case 4:
                                            issueExport.setUrl10(url);
                                            break;
                                        default:
                                            break;
                                    }
                                }
                            }
                        }
                    }
                }

                exportList.add(issueExport);
            }
        }

        return exportList;
    }
}
