package com.example.civilizedcity.common;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class StringUtils {
    private static Pattern humpPattern = Pattern.compile("[A-Z]");

    public static boolean isEmpty(String str) {
        return null == str || "".equals(str.trim()) || "null".equals(str) || "undefined".equals(str);
    }
    public static boolean isNotEmpty(String str) {
        return !isEmpty(str);
    }

    public static String trim(String src, String...st) {
        if (null == src) return src;
        if (null == st || st.length == 0) return src.trim();

        String start = st[0];
        if (!src.startsWith(start)) {
            return src;
        }

        return src.substring(start.length());
    }

    public static String trimEnd(String src, String str) {
        if (null == src || null == str) {
            return src;
        }

        if (src.endsWith(str)) {
            return src.substring(0, src.length() - str.length());
        }

        return src;
    }

    public static String ifNull(String src, String defaultVal) {
        return isEmpty(src) ? defaultVal : src;
    }

    public static String random(int length) {
        String str="abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for(int i=0;i<length;i++){
            int number=random.nextInt(62);
            sb.append(str.charAt(number));
        }
        return sb.toString();
    }

    public static String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    public static String urlEncode(String str) {
        if (isEmpty(str)) return str;

        try {
            return URLEncoder.encode(str, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            return str;
        }
    }

    public static String urlDecode(String str) {
        if (isEmpty(str)) return str;

        try {
            return URLDecoder.decode(str, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            return str;
        }
    }

    public static String encodeURIComponent(String str) {
        String result = urlEncode(str);

        return result.replaceAll("\\+", "%20")
                .replaceAll("\\%21", "!")
                .replaceAll("\\%27", "'")
                .replaceAll("\\%28", "(")
                .replaceAll("\\%29", ")")
                .replaceAll("\\%7E", "~");
    }

    public static String humpToLine(String str) {
        if (null == str || "".equals(str)) return "";

        Matcher matcher = humpPattern.matcher(str);
        StringBuffer sb = new StringBuffer();
        while (matcher.find()) {
            matcher.appendReplacement(sb, "_" + matcher.group(0).toLowerCase());
        }
        matcher.appendTail(sb);
        return sb.toString();
    }

    public static String strToUnicode(String str) {
        char[] chars = str.toCharArray();
        String returnStr = "";
        for (int i = 0; i < chars.length; i++) {
            returnStr += "\\u" + Integer.toString(chars[i], 16);
        }
        return returnStr;
    }

    public static String repeat(String src, int len) {
        if (null == src) {
            return null;
        }

        if (len <= 0) {
            return src;
        }

        String res = "";
        for (int i = 0; i < len; i ++) {
            res += src;
        }

        return res;
    }

    public static String lpad(String src, String padStr, int len) {
        if (null == src) {
            return null;
        }

        if (len <= 0 || src.length() >= len) {
            return src;
        }

        if (null == padStr) {
            padStr = "";
        }

        int padLen = len - src.length();
        String prefix = repeat(padStr, padLen);
        prefix = prefix.substring(0, padLen);
        return prefix + src;
    }

    /**
     * 解析 QueryString
     * @param queryStr
     * @return
     */
    public static Map<String, String> parseQueryString(String queryStr) {
        if (isEmpty(queryStr)) {
            return null;
        }
        Map<String, String> result = new HashMap<String, String>();

        String[] fields = queryStr.split("&");
        for (String field : fields) {
            String[] pairs = field.split("=");
            if (2 != pairs.length) {
                continue;
            }

            try {
                String key = URLDecoder.decode(pairs[0].trim(), "UTF-8");
                String val = URLDecoder.decode(pairs[1].trim(), "UTF-8");

                result.put(key, val);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }

        return result;
    }

    /**
     * Map 转 QueryString
     * @param params
     * @return
     */
    public static String buildQueryString(Map<String, Object> params) {
        if (null == params) {
            return null;
        }

        List<String> lst = new ArrayList<>();
        for (Map.Entry<String, Object> entry : params.entrySet()) {
            String key = StringUtils.urlEncode(entry.getKey());
            Object o = entry.getValue();
            if (null == o) {
                continue;
            }
            String val = o.toString();
            if (StringUtils.isEmpty(val)) {
                continue;
            }
            String queryStr = key + "=" + StringUtils.urlEncode(val);
            lst.add(queryStr);
        }
        return String.join("&", lst);
    }
}
