package com.example.civilizedcity.config;

import cn.dev33.satoken.context.SaHolder;
import cn.dev33.satoken.filter.SaServletFilter;
import cn.dev33.satoken.jwt.StpLogicJwtForStateless;
import cn.dev33.satoken.router.SaHttpMethod;
import cn.dev33.satoken.router.SaRouter;
import cn.dev33.satoken.stp.StpLogic;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.json.JSONUtil;
import com.example.civilizedcity.common.ResponseBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class SaTokenConfigure {

    @Autowired
    FilterConfig filterConfig;

    @Bean
    public SaServletFilter getSaServletFilter() {
        SaServletFilter saServletFilter = new SaServletFilter();
        saServletFilter.addInclude("/**");
        saServletFilter.addExclude(filterConfig.getAnnList().toArray(new String[0]));
        saServletFilter.setAuth(auth -> {
            // OPTIONS 请求不校验, 主要是为了解决跨域问题
            SaRouter.notMatch(SaHttpMethod.OPTIONS).check(r -> StpUtil.checkLogin());
        });

        saServletFilter.setError(err -> {
            SaHolder.getResponse().setHeader("Content-Type", "application/json;charset=UTF-8");
            SaHolder.getResponse().setStatus(200);
            String jsonStr = JSONUtil.toJsonStr(ResponseBean.error(err.getMessage(), ResponseBean.ERROR_AUTH_FAIL));
            return jsonStr;
        });

        return saServletFilter;
    }


    // Sa-Token 整合 jwt (Style模式)
    @Bean
    public StpLogic getStpLogicJwt() {
        return new StpLogicJwtForStateless();
    }
}
