package com.example.civilizedcity.controller;

import cn.dev33.satoken.stp.StpUtil;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.vo.ChangePassword;
import com.example.civilizedcity.vo.LoginParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;


@Api(tags ="登录")
@RestController
@RequestMapping("/admin")
public class LoginController extends BaseController {

    @PostMapping("/login")
    @ApiOperation(value="后台登录", notes = "后台登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestBody LoginParam loginParm) throws Exception {

        if (StringUtils.isEmpty(loginParm.getAccount()) || StringUtils.isEmpty(loginParm.getPassword())) {
            return ResponseBean.error("登录用户或者密码不能为空");
        }

        SysUser sysUser = sysUserService.login(loginParm);

        StpUtil.login(sysUser.getUserId(), "admin");
        String tokenValue = StpUtil.getTokenValue();

        Map<String, Object> result = new HashMap<>();
        result.put("user", sysUser);
        result.put("token", tokenValue);

        return ResponseBean.success(result);
    }

    @PostMapping("/logout")
    @ApiOperation(value="登出", notes = "登出", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean logout() throws Exception {
        SysUser sysUser = currentUser();
        StpUtil.logout(sysUser.getUserId(), "admin");
        return ResponseBean.success("success");
    }


    @PutMapping("/change-password")
    @ApiOperation(value="修改密码", notes = "修改密码", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean changePassword(@ApiParam("修改密码参数") @RequestBody ChangePassword param) throws Exception {
        if (StringUtils.isEmpty(param.getOriginPassword()) || StringUtils.isEmpty(param.getNewPassword())) {
            return ResponseBean.error("原始密码或新密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        SysUser user = currentUser();

        boolean isOk = sysUserService.changePassword(user, param);
        if (!isOk) {
            ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success("密码修改成功");
    }
}
