package com.example.civilizedcity.controller;

import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaPerson;
import com.example.civilizedcity.service.TaIssueService;
import com.example.civilizedcity.service.TsStatDataService;
import com.example.civilizedcity.vo.StatVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 查询统计
 */
@Api(tags = "常用统计对象功能接口")
@RestController
public class StatController extends BaseController {

    @Autowired
    TaIssueService taIssueService;

    @Autowired
    TsStatDataService tsStatDataService;

    @ApiOperation("小程序首页查询统计")
    @GetMapping("/ma/index-stat")
    public ResponseBean getMaIndex(@ApiParam("身份") @RequestParam(value = "duty", required = false) String duty) throws Exception {
        SysUser user = null;
        TaPerson person = null;

        try {
            user = currentUser();
        } catch (Exception e) {
            //
        }

        person = currentPerson();
        if (null == person) {
            return ResponseBean.error("请在小程序端操作");
        }

        List<StatVo> result = taIssueService.statMaIndex(user, person, duty);

        return ResponseBean.success(result);
    }

    @ApiOperation("管理端首页统计")
    @GetMapping("/stat/admin/index-banner")
    public ResponseBean statAdminIndexBanner() throws Exception {
        Map<String, Object> result = tsStatDataService.statAdminIndexBanner();
        return ResponseBean.success(result);
    }

    @ApiOperation("问题单数量查询统计")
    @GetMapping("/stat/issue")
    public ResponseBean statIssueData(@ApiParam("统计类型") @RequestParam String statType,
                                      @ApiParam("开始日期") @RequestParam String startDate,
                                      @ApiParam("结束日期") @RequestParam String endDate,
                                      @ApiParam("统计条数") @RequestParam Integer num) throws Exception {
        List<StatVo> result = tsStatDataService.statIssue(statType, startDate, endDate, num);
        return ResponseBean.success(result);
    }

    @ApiOperation("单位交办统计")
    @GetMapping("/stat/issue/org")
    public ResponseBean statIssueOrg(@ApiParam("单位ID") @RequestParam String orgId) throws Exception {
        List<StatVo> result = tsStatDataService.statIssueOrg(orgId);
        return ResponseBean.success(result);
    }

    @ApiOperation("问题分类统计")
    @GetMapping("/stat/issue/type")
    public ResponseBean statIssueType() throws Exception {
        List<StatVo> result = tsStatDataService.statIssueType();
        return ResponseBean.success(result);
    }

    @ApiOperation("点位分类统计")
    @GetMapping("/stat/issue/loc")
    public ResponseBean statLoc() throws Exception {
        List<StatVo> result = tsStatDataService.statIssueLoc();
        return ResponseBean.success(result);
    }
}
