package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ExcelUtils;
import com.example.civilizedcity.common.ResponseBean;

import java.util.List;

import com.example.civilizedcity.vo.CheckLoc;
import com.example.civilizedcity.vo.CheckLocAnswer;
import com.example.civilizedcity.vo.CheckSurvey;
import com.example.civilizedcity.vo.CheckSurveyAnswer;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaCheckAnswer;
import com.example.civilizedcity.service.TaCheckAnswerService;

import javax.servlet.http.HttpServletResponse;

/**
 * 测评作答;(ta_check_answer)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-13
 */
@Api(tags = "测评作答对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckAnswerController extends BaseController {

    @Autowired
    private TaCheckAnswerService taCheckAnswerService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckAnswer/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taCheckAnswerService.getById(id));
    }

    /**
     * 分页查询答题详情
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询答题详情")
    @GetMapping("/taCheckAnswer")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("答卷类型") @RequestParam(value = "itemType") String itemType,
                             @ApiParam("测评ID") @RequestParam(value = "checkId") String checkId,
                             @ApiParam("点位ID") @RequestParam(value = "typeId", required = false) String typeId,
                             @ApiParam("社区") @RequestParam(value = "communityName", required = false) String communityName,
                             @ApiParam("答卷类型") @RequestParam(value = "addr", required = false) String addr,
                             @ApiParam("答题人") @RequestParam(value = "userName", required = false) String userName) throws Exception {

        IPage<TaCheckAnswer> page = new Page<>(pageNum, pageSize);

        page = taCheckAnswerService.getPageBy(page, checkId, itemType, typeId, communityName, addr, userName);

        return ResponseBean.success(page);
    }


    /**
     * 答题详情导出
     *
     * @return 查询结果
     */
    @ApiOperation("答题详情导出")
    @PostMapping("/taCheckAnswer/export")
    public ResponseBean export(@ApiParam("答卷类型") @RequestParam(value = "itemType") String itemType,
                               @ApiParam("测评ID") @RequestParam(value = "checkId") String checkId,
                               @ApiParam("点位ID") @RequestParam(value = "typeId", required = false) String typeId,
                               @ApiParam("社区") @RequestParam(value = "communityName", required = false) String communityName,
                               @ApiParam("答卷类型") @RequestParam(value = "addr", required = false) String addr,
                               @ApiParam("答题人") @RequestParam(value = "userName", required = false) String userName,
                               HttpServletResponse response) throws Exception {

        if (Constants.CHECK_OF_SURVEY.equals(itemType)) {
            List<CheckSurvey> list = taCheckAnswerService.getList2By(checkId, itemType, typeId, communityName, addr, userName);
            ExcelUtils.flush(response, CheckSurvey.class, list, "问卷调查答卷");
        } else {
            List<CheckLoc> list = taCheckAnswerService.getListBy(checkId, itemType, typeId, communityName, addr, userName);
            ExcelUtils.flush(response, CheckLoc.class, list, "实地测评答卷");
        }

        return null;
    }

    /**
     * 分页查询答题详情
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询答题详情")
    @GetMapping("/taCheckAnswer/{answerId}/detail")
    public ResponseBean listDetail(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                   @ApiParam("答案ID") @PathVariable String answerId,
                                   @ApiParam("答卷类型") @RequestParam(value = "itemType", required = false) String itemType,
                                   @ApiParam("测评ID") @RequestParam(value = "checkId", required = false) String checkId,
                                   @ApiParam("点位ID") @RequestParam(value = "typeId", required = false) String typeId,
                                   @ApiParam("社区") @RequestParam(value = "communityName", required = false) String communityName,
                                   @ApiParam("答卷类型") @RequestParam(value = "addr", required = false) String addr) throws Exception {

        if (Constants.CHECK_OF_LOC.equals(itemType)) {
            IPage<CheckLocAnswer> pg = new Page<>(pageNum, pageSize);
            IPage<CheckLocAnswer> result = taCheckAnswerService.getLocPageBy(pg, checkId, typeId, answerId);
            return ResponseBean.success(result);
        }

        if (Constants.CHECK_OF_SURVEY.equals(itemType)) {
            IPage<CheckSurveyAnswer> pg = new Page<>(pageNum, pageSize);
            IPage<CheckSurveyAnswer> result = taCheckAnswerService.getSurveyPageBy(pg, checkId, answerId, communityName, addr);
            return ResponseBean.success(result);
        }

        return ResponseBean.error("非法的答卷类型");
    }

    /**
     * 导出答题详情
     *
     * @return 查询结果
     */
    @ApiOperation("导出答题详情")
    @GetMapping("/taCheckAnswer/{answerId}/detail/export")
    public ResponseBean exportDetail(
            @ApiParam("答案ID") @PathVariable String answerId,
            @ApiParam("答卷类型") @RequestParam(value = "itemType", required = false) String itemType,
            @ApiParam("测评ID") @RequestParam(value = "checkId", required = false) String checkId,
            @ApiParam("点位ID") @RequestParam(value = "typeId", required = false) String typeId,
            @ApiParam("社区") @RequestParam(value = "communityName", required = false) String communityName,
            @ApiParam("答卷类型") @RequestParam(value = "addr", required = false) String addr,
            HttpServletResponse response) throws Exception {

        if (Constants.CHECK_OF_LOC.equals(itemType)) {
            List<CheckLocAnswer> result = taCheckAnswerService.getLocListBy(checkId, typeId, answerId);
            ExcelUtils.flush(response, CheckLocAnswer.class, result, "实地测评答案明细");
            return null;
        }

        if (Constants.CHECK_OF_SURVEY.equals(itemType)) {
            List<CheckSurveyAnswer> result = taCheckAnswerService.getSurveyListBy(checkId, answerId, communityName, addr);
            ExcelUtils.flush(response, CheckSurveyAnswer.class, result, "调查问卷答案明细");
            return null;
        }

        return ResponseBean.error("非法的答卷类型");
    }

    /**
     * 新增数据
     *
     * @param taCheckAnswer 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckAnswer")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckAnswer taCheckAnswer) throws Exception {
        taCheckAnswerService.save(taCheckAnswer);
        return ResponseBean.success(taCheckAnswer);
    }

    /**
     * 更新数据
     *
     * @param taCheckAnswer 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taCheckAnswer/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheckAnswer taCheckAnswer,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taCheckAnswerService.updateById(taCheckAnswer);
        return ResponseBean.success(taCheckAnswer);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheckAnswer/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taCheckAnswerService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}