package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.util.List;

import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaAttach;
import com.example.civilizedcity.service.TaAttachService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaCheckAnswerItem;
import com.example.civilizedcity.service.TaCheckAnswerItemService;

/**
 * 测评作答详情;(ta_check_answer_item)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-13
 */
@Api(tags = "测评作答详情对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckAnswerItemController extends BaseController {

    @Autowired
    private TaCheckAnswerItemService taCheckAnswerItemService;

    @Autowired
    private TaAttachService taAttachService;

    /**
     * 通过ID查询单条数据
     *
     * @param itemId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckAnswerItem/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taCheckAnswerItemService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param quId  问题ID
     * @param isMine 我的
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheckAnswerItem")
    public ResponseBean list(@ApiParam("问题ID") @RequestParam(value = "quId") String quId,
                             @ApiParam("我的") @RequestParam(value = "isMine") Boolean isMine) throws Exception {

        SysUser sysUser = currentUser();
        List<TaCheckAnswerItem> result = taCheckAnswerItemService.getListBy(quId, isMine, sysUser);

        if (null != result && result.size() > 0) {
            for(TaCheckAnswerItem item : result) {
                List<TaAttach> attachList = taAttachService.getListBy(Constants.SOURCE_CHECK_ANSWER, item.getItemId());
                item.setAttachList(attachList);
            }
        }

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taCheckAnswerItem 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckAnswerItem")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckAnswerItem taCheckAnswerItem) throws Exception {
        taCheckAnswerItemService.save(taCheckAnswerItem);
        return ResponseBean.success(taCheckAnswerItem);
    }

    /**
     * 更新数据
     *
     * @param taCheckAnswerItem 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taCheckAnswerItem/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheckAnswerItem taCheckAnswerItem,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taCheckAnswerItemService.updateById(taCheckAnswerItem);
        return ResponseBean.success(taCheckAnswerItem);
    }

    /**
     * 通过主键删除数据
     *
     * @param itemId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheckAnswerItem/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taCheckAnswerItemService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}