package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;

import java.util.List;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaCheckItemAn;
import com.example.civilizedcity.service.TaCheckItemAnService;

/**
 * 项目问题答案;(ta_check_item_an)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-16
 */
@Api(tags = "项目问题答案对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckItemAnController extends BaseController {

    @Autowired
    private TaCheckItemAnService taCheckItemAnService;

    /**
     * 通过ID查询单条数据
     *
     * @param answerId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckItemAn/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taCheckItemAnService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param quId 问题ID
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheckItemAn")
    public ResponseBean list(@ApiParam("问题ID") @RequestParam(value = "quId") String quId) throws Exception {

        QueryWrapper<TaCheckItemAn> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("qu_id", quId);
        queryWrapper.orderByAsc("sort_no");
        List<TaCheckItemAn> result = taCheckItemAnService.list(queryWrapper);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taCheckItemAn 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckItemAn")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckItemAn taCheckItemAn) throws Exception {
        taCheckItemAnService.save(taCheckItemAn);
        return ResponseBean.success(taCheckItemAn);
    }

    /**
     * 更新数据
     *
     * @param taCheckItemAn 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taCheckItemAn/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheckItemAn taCheckItemAn,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taCheckItemAnService.updateById(taCheckItemAn);
        return ResponseBean.success(taCheckItemAn);
    }

    /**
     * 通过主键删除数据
     *
     * @param answerId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheckItemAn/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taCheckItemAnService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}