package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;

import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 测评点位;(ta_check_item)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-13
 */
@Api(tags = "测评点位对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckItemController extends BaseController {

    @Autowired
    private TaCheckItemService taCheckItemService;

    @Autowired
    private TdLocTypeService tdLocTypeService;

    @Autowired
    private TaCheckService taCheckService;

    @Autowired
    private TaCheckAnswerService taCheckAnswerService;

    @Autowired
    private TaCheckItemQuService taCheckItemQuService;

    @Autowired
    private TaCheckAnswerItemService taCheckAnswerItemService;

    /**
     * 通过ID查询单条数据
     *
     * @param itemId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckItem/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaCheckItem taCheckItem = taCheckItemService.getById(id);
        if (null == taCheckItem || Constants.STATUS_DELETE == taCheckItem.getStatus()) {
            return ResponseBean.error("未找到测评项目");
        }

        return ResponseBean.success(taCheckItem);
    }

    /**
     * 通过测评以及点位查询数据
     *
     * @param checkId 测评ID
     * @param typeId  点位ID
     * @return 实例对象
     */
    @ApiOperation("通过测评以及点位查询数据")
    @GetMapping("/taCheck/{checkId}/item/{typeId}")
    public ResponseBean queryByCheckId(@ApiParam("测评ID") @PathVariable String checkId,
                                       @ApiParam("点位ID 或者 调查问卷") @PathVariable String typeId) throws Exception {
        String itemType = StringUtils.isEmpty(typeId) || Constants.CHECK_OF_SURVEY.equals(typeId) ? Constants.CHECK_OF_SURVEY : Constants.CHECK_OF_LOC;
        TaCheckItem taCheckItem = getByCheckAndType(checkId, typeId, itemType);
        if (null == taCheckItem || Constants.STATUS_DELETE == taCheckItem.getStatus()) {
            return ResponseBean.error("未找到测评项目");
        }

        if (Constants.CHECK_OF_SURVEY.equals(typeId)) {
            SysUser sysUser = currentUser();
            taCheckItem.setAnswerNum(taCheckAnswerService.getCountByUser(taCheckItem.getItemId(), sysUser.getUserId()));
        }

        return ResponseBean.success(taCheckItem);
    }

    private TaCheckItem getByCheckAndType(String checkId, String typeId, String itemType) {
        QueryWrapper<TaCheckItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("check_id", checkId);

        if (Constants.CHECK_OF_SURVEY.equals(itemType)) {
            queryWrapper.eq("item_type", Constants.CHECK_OF_SURVEY);
        } else {
            queryWrapper.eq("item_type", Constants.CHECK_OF_LOC); // 只查点位
            queryWrapper.eq("type_id", typeId);
        }
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.last("limit 1");

        return taCheckItemService.getOne(queryWrapper);
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheckItem")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("模拟测评ID") @RequestParam(value = "checkId") String checkId,
                             @ApiParam("项目类型") @RequestParam(value = "itemType") String itemType) throws Exception {

        SysUser sysUser = currentUser();
        IPage<TaCheckItem> pg = new Page<>(pageNum, pageSize);
        IPage<TaCheckItem> result = taCheckItemService.getPageBy(pg, checkId, itemType);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taCheckItem 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckItem")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckItem taCheckItem) throws Exception {

        if (StringUtils.isEmpty(taCheckItem.getItemType())) {
            return ResponseBean.error("测评类型不能为空");
        }

        if (Constants.CHECK_OF_LOC.equals(taCheckItem.getItemType()) && StringUtils.isEmpty(taCheckItem.getTypeId())) {
            return ResponseBean.error("测评点位不能为空");
        }

        if (!StringUtils.isEmpty(taCheckItem.getTypeId())) {
            TdLocType tdLocType = tdLocTypeService.getById(taCheckItem.getTypeId());
            if (null != tdLocType) {
                taCheckItem.setName(tdLocType.getName());
            } else {
                return ResponseBean.error("测评点位选择不正确");
            }
        }

        TaCheckItem origin = getByCheckAndType(taCheckItem.getCheckId(), taCheckItem.getTypeId(), taCheckItem.getItemType());
        if (null == origin) {
            // 新增
            taCheckItem.setStatus(Constants.STATUS_NORMAL);
            taCheckItem.setCreateDate(LocalDateTime.now());
            taCheckItemService.save(taCheckItem);
        } else {
            // 更新
            taCheckItem.setItemId((origin.getItemId()));
            taCheckItemService.updateById(taCheckItem);
        }

        return ResponseBean.success(taCheckItem);
    }

    @ApiOperation("答卷检测")
    @PutMapping("/taCheckItem/{id}/answer/pre-check")
    public ResponseBean checkAnswer(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaCheckItem taCheckItem = taCheckItemService.getById(id);
        if (null == taCheckItem || taCheckItem.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.success(new HashMap<String, Object>(){{
                put("errorCode", -1);   // 负数代表不能进行任何操作, 包括查看
                put("message", "未找到有效的答题信息");
            }});
        }

        TaCheck taCheck = taCheckService.getById(taCheckItem.getCheckId());
        if (null == taCheck || taCheck.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.success(new HashMap<String, Object>(){{
                put("errorCode", -2);
                put("message", "未找到有效的测评信息");
            }});
        }

        if (LocalDateTime.now().isAfter(DateUtils.day2LocalDateime2(taCheck.getEndDate()))) {
            return ResponseBean.success(new HashMap<String, Object>(){{
                put("errorCode", 1);    // 正数代表, 只能查看
                put("message", "本次测评已结束");
            }});
        }

//        if (Constants.CHECK_OF_LOC.equals(taCheckItem.getItemType())) {
            int cnt = taCheckAnswerService.countByItem(taCheckItem.getItemId());
            if (cnt >= taCheckItem.getNum()) {
                return ResponseBean.success(new HashMap<String, Object>(){{
                    put("errorCode", 3);
                    put("message", "当前问卷已收齐, 无须作答");
                }});
            }
//        }

        return ResponseBean.success(new HashMap<String, Object>(){{
            put("errorCode", 0);    // 0 代表可以答题
            put("message", null);
        }});
    }

    /**
     * 获取回答
     *
     * @param id 实例对象
     * @return 实例对象
     */
    @ApiOperation("获取回答")
    @GetMapping("/taCheckItem/{id}/answer")
    public ResponseBean getAnswer(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        SysUser sysUser = currentUser();
        List<TaCheckAnswer> answerList = taCheckAnswerService.getByUser(id, sysUser.getUserId());
        return ResponseBean.success(answerList);
    }

    /**
     * 回答问题
     *
     * @param taCheckAnswer 实例对象
     * @return 实例对象
     */
    @ApiOperation("回答问题")
    @PutMapping("/taCheckItem/{id}/answer")
    public ResponseBean answer(@ApiParam("对象实体") @RequestBody TaCheckAnswer taCheckAnswer,
                               @ApiParam("对象ID") @PathVariable String id) throws Exception {

        SysUser sysUser = currentUser();
        TaCheckItem taCheckItem = taCheckItemService.getById(id);
        if (null == taCheckItem || taCheckItem.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到有效的答题信息");
        }

        TaCheck taCheck = taCheckService.getById(taCheckItem.getCheckId());
        if (null == taCheck || taCheck.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到有效的测评信息");
        }

        if (LocalDateTime.now().isAfter(DateUtils.day2LocalDateime2(taCheck.getEndDate()))) {
            return ResponseBean.error("本次测评已结束");
        }

        if (StringUtils.isEmpty(taCheckAnswer.getAddr())) {
            return ResponseBean.error("测评地点详情或小区名称未填写");
        }

        // 暂时不限制一个人答多少份
//        int count = taCheckAnswerService.getCountByUser(id, sysUser.getUserId());
//        if (count > 0) {
//            return ResponseBean.error("您已完成测评作答");
//        }

        taCheckAnswer.setAnswerId(StringUtils.uuid());
        taCheckAnswer.setCheckId(taCheck.getCheckId());
        taCheckAnswer.setItemId(id);
        taCheckAnswer.setStatus(Constants.STATUS_NORMAL);
        taCheckAnswer.setOrgId(sysUser.getOrgId());
        taCheckAnswer.setCreateUser(sysUser.getUserId());
        taCheckAnswer.setUserName(sysUser.getName());
        taCheckAnswer.setCreateDate(LocalDateTime.now());

        taCheckItemService.answer(taCheckItem, taCheckAnswer, sysUser);
        return ResponseBean.success(taCheckAnswer);
    }

    /**
     * 通过主键删除数据
     *
     * @param itemId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheckItem/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taCheckItemService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}