package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;

import java.util.List;

import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.TaCheckItemAn;
import com.example.civilizedcity.service.TaCheckItemAnService;
import com.example.civilizedcity.service.TaCheckItemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaCheckItemQu;
import com.example.civilizedcity.service.TaCheckItemQuService;

/**
 * 点位问题;(ta_check_item_qu)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-13
 */
@Api(tags = "点位问题对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckItemQuController extends BaseController {

    @Autowired
    private TaCheckItemService taCheckItemService;

    @Autowired
    private TaCheckItemQuService taCheckItemQuService;

    @Autowired
    private TaCheckItemAnService taCheckItemAnService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckItemQu/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        return ResponseBean.success(taCheckItemQuService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheckItemQu")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("点位ID") @RequestParam(value = "itemId") String itemId) throws Exception {

        IPage<TaCheckItemQu> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaCheckItemQu> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("item_id", itemId);
        queryWrapper.orderByAsc("sort_no");
        IPage<TaCheckItemQu> result = taCheckItemQuService.page(pg, queryWrapper);

        List<TaCheckItemQu> quList = result.getRecords();
        if (quList != null && quList.size() > 0) {
            for (TaCheckItemQu item : quList) {
                List<TaCheckItemAn> answerList = taCheckItemAnService.getListByQuId(item.getQuId());
                item.setAnswerList(answerList);
            }
        }

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taCheckItemQu 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckItemQu")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckItemQu taCheckItemQu) throws Exception {
        String quId = taCheckItemQu.getQuId();

        if (StringUtils.isEmpty(quId)) {
            // 新增
//            int maxNo = taCheckItemQuService.getMaxNoBy(taCheckItemQu.getItemId());
//            taCheckItemQu.setSortNo(maxNo + 1);
            taCheckItemQuService.save(taCheckItemQu);
        } else {
            taCheckItemQuService.updateById(taCheckItemQu);
        }

        taCheckItemAnService.mergeData(taCheckItemQu.getQuId(), taCheckItemQu.getAnswerList());

        taCheckItemService.updateFullScore(taCheckItemQu.getItemId());

        return ResponseBean.success(taCheckItemQu);
    }

//    /**
//     * 更新数据
//     *
//     * @param taCheckItemQu 实例对象
//     * @return 实例对象
//     */
//    @ApiOperation("更新数据")
//    @PutMapping("/taCheckItemQu/{id}")
//    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheckItemQu taCheckItemQu,
//                            @ApiParam("对象ID") @PathVariable Integer id ) throws Exception {
//        taCheckItemQuService.updateById(taCheckItemQu);
//        return ResponseBean.success(taCheckItemQu);
//    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheckItemQu/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        taCheckItemQuService.removeById(id);
        return ResponseBean.success("success");
    }
}