package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;
import java.util.List;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaCheckScore;
import com.example.civilizedcity.service.TaCheckScoreService;

 /**
 * 测评分值设置;(ta_check_score)表控制层
 * @author : http://njyunzhi.com
 * @date : 2022-12-13
 */
@Api(tags = "测评分值设置对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckScoreController extends BaseController {
    
    @Autowired
    private TaCheckScoreService taCheckScoreService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckScore/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        return ResponseBean.success(taCheckScoreService.getById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheckScore")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception {
        
        IPage<TaCheckScore> pg = new Page<>(pageNum, pageSize);
        // QueryWrapper<TaCheckScore> queryWrapper = new QueryWrapper<>();
        // queryWrapper.orderByDesc("create_date");
        IPage<TaCheckScore> result = taCheckScoreService.page(pg);
        
        return ResponseBean.success(result);
    }
    
    /** 
     * 新增数据
     *
     * @param taCheckScore 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckScore")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckScore taCheckScore) throws Exception {
        taCheckScoreService.save(taCheckScore);
        return ResponseBean.success(taCheckScore);
    }
    
    /** 
     * 更新数据
     *
     * @param taCheckScore 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taCheckScore/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheckScore taCheckScore,
                            @ApiParam("对象ID") @PathVariable Integer id ) throws Exception {
        taCheckScoreService.updateById(taCheckScore);
        return ResponseBean.success(taCheckScore);
    }
    
    /** 
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taCheckScore/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id){
        taCheckScoreService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}