package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.util.List;

import com.example.civilizedcity.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaCheckStand;
import com.example.civilizedcity.service.TaCheckStandService;

/**
 * 测评标准;(ta_check_stand)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2023-1-29
 */
@Api(tags = "测评标准对象功能接口")
@RestController
@RequestMapping("/")
public class TaCheckStandController extends BaseController {

    @Autowired
    private TaCheckStandService taCheckStandService;

    /**
     * 通过ID查询单条数据
     *
     * @param statndId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taCheckStand/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taCheckStandService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taCheckStand")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("点位ID") @RequestParam(value = "locId", required = false) String locId) throws Exception {

        IPage<TaCheckStand> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaCheckStand> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(locId), "loc_id", locId);
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByDesc("create_date");
        IPage<TaCheckStand> result = taCheckStandService.page(pg);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param taCheckStand 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taCheckStand")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaCheckStand taCheckStand) throws Exception {
        taCheckStandService.save(taCheckStand);
        return ResponseBean.success(taCheckStand);
    }

    /**
     * 更新数据
     *
     * @param taCheckStand 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taCheckStand/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaCheckStand taCheckStand,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        taCheckStand.setStatndId(id);
        taCheckStandService.updateById(taCheckStand);
        return ResponseBean.success(taCheckStand);
    }
//
//    /**
//     * 通过主键删除数据
//     *
//     * @param statndId 主键
//     * @return 是否成功
//     */
//    @ApiOperation("通过主键删除数据")
//    @DeleteMapping("/taCheckStand/{id}")
//    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id){
//        taCheckStandService.removeLogicById(id);
//        return ResponseBean.success("success");
//    }
}