package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;
import java.util.List;

import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.TaAttach;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.entity.TaPerson;
import com.example.civilizedcity.service.TaAttachService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaFeedback;
import com.example.civilizedcity.service.TaFeedbackService;

 /**
 * 反馈表;(ta_feedback)表控制层
 * @author : http://njyunzhi.com
 * @date : 2023-1-30
 */
@Api(tags = "反馈表对象功能接口")
@RestController
@RequestMapping("/")
public class TaFeedbackController extends BaseController {
    
    @Autowired
    private TaFeedbackService taFeedbackService;

    @Autowired
    private TaAttachService taAttachService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taFeedback/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaFeedback taFeedback = taFeedbackService.getById(id);
        List<TaAttach> attachList = taAttachService.getListBy(Constants.SOURCE_FEEDBACK, taFeedback.getFeedbackId());
        taFeedback.setAttachList(attachList);
        return ResponseBean.success(taFeedback);
    }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taFeedback")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                             @ApiParam("我的") @RequestParam(value ="isMine", defaultValue = "false") Boolean isMine,
                             @ApiParam("问题单状态") @RequestParam(value = "bizStatus", required = false) String bizStatus) throws Exception {

        TaPerson taPerson = currentPerson();
        if (null == taPerson && isMine) {
            return ResponseBean.error("请在小程序中操作");
        }

        String personId = isMine ? taPerson.getPersonId() : null;
        
        IPage<TaIssue> pg = new Page<>(pageNum, pageSize);
        IPage<TaIssue> result = taFeedbackService.getPageBy(pg, personId, bizStatus);
        
        return ResponseBean.success(result);
    }
    
    /** 
     * 新增数据
     *
     * @param taFeedback 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taFeedback")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaFeedback taFeedback) throws Exception {

        if (StringUtils.isEmpty(taFeedback.getContent())) {
            return ResponseBean.error("请填写反馈内容");
        }
        if (StringUtils.isEmpty(taFeedback.getAddr())) {
            return ResponseBean.error("请填写具体位置");
        }

        if (null == taFeedback.getAttachList() || taFeedback.getAttachList().size() == 0) {
            return ResponseBean.error("请上传照片");
        }

        TaPerson taPerson = currentPerson();

        taFeedbackService.createNew(taFeedback, taPerson);
        return ResponseBean.success(taFeedback);
    }
    
//    /**
//     * 更新数据
//     *
//     * @param taFeedback 实例对象
//     * @return 实例对象
//     */
//    @ApiOperation("更新数据")
//    @PutMapping("/taFeedback/{id}")
//    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaFeedback taFeedback,
//                            @ApiParam("对象ID") @PathVariable String id ) throws Exception {
//        taFeedbackService.updateById(taFeedback);
//        return ResponseBean.success(taFeedback);
//    }
    
    /** 
     * 通过主键删除数据
     *
     * @param feedbackId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taFeedback/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaPerson taPerson = currentPerson();
        if (null == taPerson) {
            return ResponseBean.error("请在小程序中操作");
        }

        TaFeedback taFeedback = taFeedbackService.getById(id);
        if (null == taFeedback || taFeedback.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到相关数据");
        }

        if (!taPerson.getPersonId().equals(taFeedback.getPersonId())) {
            return ResponseBean.error("暂无权限");
        }

        taFeedbackService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}