package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.service.TaIssueService;
import com.example.civilizedcity.service.TaOrgIssueService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaIssueApply;
import com.example.civilizedcity.service.TaIssueApplyService;

/**
 * 问题处理申请;(ta_issue_apply)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-27
 */
@Api(tags = "问题处理申请对象功能接口")
@RestController
@RequestMapping("/")
public class TaIssueApplyController extends BaseController {

    @Autowired
    private TaIssueApplyService taIssueApplyService;

    @Autowired
    private TaIssueService taIssueService;

    @Autowired
    private TaOrgIssueService taOrgIssueService;

    @Autowired
    ApplicationEventPublisher applicationEventPublisher;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taIssueApply/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        return ResponseBean.success(taIssueApplyService.getById(id));
    }

    private String getOrgId() throws Exception {
        SysUser sysUser = currentUser();
        String orgId = isManager(sysUser) ? null : sysUser.getOrgId();
        String duty = getPersonDuty();

        if (StringUtils.isNotEmpty((duty))) {
            if (Constants.DUTY_ORG_MANAGER.equals(duty) || Constants.DUTY_ORG_USER.equals(duty)) {
                if (StringUtils.isEmpty(orgId)) {
                    orgId = sysUser.getOrgId();
                }
            }
        }

        return orgId;
    }

    /**
     * 获取申请条数
     *
     * @param state 申请状态 0, 未审批, 1 已审批
     * @return 实例对象
     */
    @ApiOperation("获取申请条数")
    @GetMapping("/taIssueApply/nums")
    public ResponseBean countApply(@ApiParam("申请状态") @RequestParam(value = "state", defaultValue = "0") Integer state,
                                   @ApiParam("申请类型") @RequestParam(value = "applyType", required = false) String applyType) throws Exception {

        String orgId = getOrgId();

        QueryWrapper<TaIssueApply> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(applyType), "apply_type", applyType);
        queryWrapper.eq(StringUtils.isNotEmpty(orgId), "org_id", orgId);

        if (0 == state) {
            queryWrapper.isNull("verify_date");
        } else {
            queryWrapper.isNotNull("verify_date");
        }

        long num = taIssueApplyService.count(queryWrapper);

        return ResponseBean.success(num);
    }

    /**
     * 通过ISSUE_ID查询单条数据
     *
     * @param issueId 问题单ID
     * @param applyType 申请类型
     * @return 实例对象
     */
    @ApiOperation("通过ISSUE_ID查询单条数据")
    @GetMapping("/taIssue/{issueId}/apply")
    public ResponseBean queryByIssueId(@ApiParam("问题单ID") @PathVariable Integer issueId,
                                       @ApiParam("申请类型") @RequestParam("applyType") String applyType) throws Exception {

        String orgId = getOrgId();

        QueryWrapper<TaIssueApply> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("issue_id", issueId);
        queryWrapper.eq("apply_type", applyType);
        queryWrapper.eq(StringUtils.isNotEmpty(orgId), "org_id", orgId);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");
        queryWrapper.last("limit 1");

        TaIssueApply taIssueApply = taIssueApplyService.getOne(queryWrapper);

        return ResponseBean.success(taIssueApply);
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taIssueApply")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("问题单ID") @RequestParam(value = "issueId", required = false) Integer issueId,
                             @ApiParam("申请类型") @RequestParam(value = "applyType", required = false) String applyType,
                             @ApiParam("问题单来源") @RequestParam(value = "sourceType", required = false) String sourceType,
                             @ApiParam(value = "是否全部") @RequestParam(value = "isAll", defaultValue = "false") Boolean isAll) throws Exception {

        String orgId = getOrgId();

        IPage<TaIssueApply> pg = new Page<>(pageNum, pageSize);
        IPage<TaIssueApply> result = taIssueApplyService.getPageBy(pg, issueId, applyType, orgId, sourceType, isAll);

        return ResponseBean.success(result);
    }

    /**
     * 申请流程
     *
     * @param taIssueApply 实例对象
     * @return 实例对象
     */
    @ApiOperation("申请流程")
    @PostMapping("/taIssueApply")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaIssueApply taIssueApply) throws Exception {
        if (null == taIssueApply.getIssueId()) {
            throw new Exception("校验问题单出错");
        }

        SysUser sysUser = currentUser();
        if (null == taOrgIssueService.getByIssueAndOrg(taIssueApply.getIssueId(), sysUser.getOrgId())) {
            throw new Exception("未找到交办单位问题单");
        }

        taIssueApplyService.createNewApply(taIssueApply, sysUser);
        return ResponseBean.success(taIssueApply);
    }

    /**
     * 审批流程
     *
     * @param taIssueApply 实例对象
     * @return 实例对象
     */
    @ApiOperation("审批流程")
    @PutMapping("/taIssueApply/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaIssueApply taIssueApply,
                             @ApiParam("对象ID") @PathVariable Integer id) throws Exception {

        TaIssueApply origin = taIssueApplyService.getById(id);
        if (null == origin || Constants.STATUS_DELETE == origin.getStatus()) {
            return ResponseBean.error("未找到申请数据");
        }

        // 如果是审核驳回
        if (StringUtils.isEmpty(taIssueApply.getVerifyDesc()) && Constants.APPLY_REJECT.equals(taIssueApply.getVerifyStatus())) {
            return ResponseBean.error("请填写驳回理由");
        }

        SysUser sysUser = currentUser();
        taIssueApply.setApplyId(id);
        taIssueApplyService.verify(taIssueApply, sysUser, origin);

        // 如果单位审核通过
        // 直接提销单申请
        if (taIssueApply.getApplyType().equals(Constants.PROCESS_APPLY_VERIFY) && taIssueApply.getVerifyStatus().equals(Constants.APPLY_PASS)) {
            TaIssueApply newIssueApply = new TaIssueApply();
            newIssueApply.setIssueId(taIssueApply.getIssueId());
            newIssueApply.setApplyType(Constants.PROCESS_APPLY_END);
            newIssueApply.setContent(taIssueApply.getContent());
            newIssueApply.setRemark(taIssueApply.getRemark());
            newIssueApply.setStatus(Constants.STATUS_NORMAL);

            taIssueApplyService.createNewApply(newIssueApply, sysUser);
        }

        return ResponseBean.success(origin);
    }

//    /**
//     * 通过主键删除数据
//     *
//     * @param id 主键
//     * @return 是否成功
//     */
//    @ApiOperation("通过主键删除数据")
//    @DeleteMapping("/taIssueApply/{id}")
//    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id) {
//        taIssueApplyService.removeLogicById(id);
//        return ResponseBean.success("success");
//    }
}