package com.example.civilizedcity.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.*;

import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.service.TaIssueService;
import com.example.civilizedcity.service.TaOrgIssueService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaIssueProcess;
import com.example.civilizedcity.service.TaIssueProcessService;

/**
 * 问题处理;(ta_issue_process)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "问题处理对象功能接口")
@RestController
@RequestMapping("/")
public class TaIssueProcessController extends BaseController {

    @Autowired
    private TaIssueProcessService taIssueProcessService;

    @Autowired
    private TaIssueService taIssueService;

    @Autowired
    private TaOrgIssueService taOrgIssueService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taIssueProcess/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        return ResponseBean.success(taIssueProcessService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taIssueProcess")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("问题单ID") @RequestParam(value = "issueId") Integer issueId) throws Exception {

        IPage<TaIssueProcess> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaIssueProcess> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("issue_id", issueId);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");
        IPage<TaIssueProcess> result = taIssueProcessService.page(pg, queryWrapper);

        return ResponseBean.success(result);
    }

    private TaIssue checkIssue(TaIssueProcess taIssueProcess) throws Exception {
        if (null == taIssueProcess.getIssueId()) {
            throw new Exception("找不到对应问题");
        }

        TaIssue taIssue = taIssueService.getById(taIssueProcess.getIssueId());
        if (null == taIssue || taIssue.getStatus() == Constants.STATUS_DELETE) {
            throw new Exception("找不到对应问题");
        }

        if (Constants.PROCESS_END.equals(taIssue.getProcessNode())) {
            throw new Exception("当前问题已办结");
        }

        return taIssue;
    }

    /**
     * 交办
     *
     * @param taIssueProcess 实例对象
     * @return 实例对象
     */
    @SaCheckPermission("manager")
    @ApiOperation("交办")
    @PostMapping("/taIssueProcess/assigned")
    public ResponseBean assigned(@ApiParam("对象实体") @RequestBody TaIssueProcess taIssueProcess) throws Exception {
        TaIssue taIssue = checkIssue(taIssueProcess);

        if (StringUtils.isEmpty(taIssueProcess.getNextOrg())) {
            return ResponseBean.error("请指定交办单位");
        }
        if (StringUtils.isEmpty(taIssueProcess.getExpireDate())) {
            return ResponseBean.error("请设置截止日期");
        } else {
            if (DateUtils.isAfter(taIssueProcess.getExpireDate() + " 23:59:59")) {
                return ResponseBean.error("请设置有效的截止日期");
            }
        }

        SysUser sysUser = currentUser();
        taOrgIssueService.createNewIssue(taIssue, taIssueProcess, sysUser);
        return ResponseBean.success(taIssueProcess);
    }

    /**
     * 管理员驳回问题单
     *
     * @param taIssueProcess 实例对象
     * @return 实例对象
     */
    @SaCheckPermission("manager")
    @ApiOperation("管理员驳回问题单")
    @PostMapping("/taIssueProcess/reject")
    public ResponseBean reject(@ApiParam("对象实体") @RequestBody TaIssueProcess taIssueProcess) throws Exception {
        TaIssue taIssue = checkIssue(taIssueProcess);

        if (StringUtils.isEmpty(taIssueProcess.getProcessResult())) {
            return ResponseBean.error("请填写未驳回原因");
        }

        // 管理员驳回问题单
        taIssueService.reject(taIssue, taIssueProcess, currentUser());
        return ResponseBean.success(taIssueProcess);
    }
}