package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.service.SysOrgService;
import com.example.civilizedcity.service.TaAttachService;
import com.example.civilizedcity.vo.IssueExport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.service.TaOrgIssueService;

import javax.servlet.http.HttpServletResponse;

/**
 * 单位问题单;(ta_org_issue)表控制层
 * @author : http://njyunzhi.com
 * @date : 2022-12-28
 */
@Api(tags = "单位问题单对象功能接口")
@RestController
@RequestMapping("/")
public class TaOrgIssueController extends BaseController {
    
    @Autowired
    private TaOrgIssueService taOrgIssueService;

    @Autowired
    private TaAttachService taAttachService;

    @Autowired
    private SysOrgService sysOrgService;

    @Autowired
    private ExportIssueUtil exportIssueUtil;


    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taOrgIssue/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable Integer id) throws Exception {
        TaOrgIssue taOrgIssue = taOrgIssueService.getById(id);
        if (null == taOrgIssue || taOrgIssue.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到记录");
        }

        List<TaAttach> attachList = taAttachService.getListBy(Constants.SOURCE_ORG_ISSUE, id.toString());
        taOrgIssue.setAttachList(attachList);

        return ResponseBean.success(taOrgIssue);
    }
     /**
      * 通过ISSUE_ID查询单条数据
      *
      * @param issueId 主键
      * @return 实例对象
      */
     @ApiOperation("通过ISSUE_ID查询单条数据")
     @GetMapping("/taIssue/{issueId}/orgIssue")
     public ResponseBean queryByIssueId(@ApiParam("对象ID") @PathVariable Integer issueId,
                                        @ApiParam("申请单位") @RequestParam(value = "orgId", required = false) String orgId) throws Exception {

         SysUser sysUser = currentUser();

         QueryWrapper<TaOrgIssue> queryWrapper = new QueryWrapper<>();
         queryWrapper.eq("issue_id", issueId);
         queryWrapper.eq(!Constants.ROOT_ORG.equals(sysUser.getOrgId()), "org_id", StringUtils.isEmpty(orgId) ? sysUser.getOrgId() : orgId);
         queryWrapper.eq("status", Constants.STATUS_NORMAL);
         queryWrapper.orderByDesc("create_date");
         queryWrapper.last("limit 1");

         TaOrgIssue taOrgIssue = taOrgIssueService.getOne(queryWrapper);
         if (null != taOrgIssue) {
             List<TaAttach> attachList = taAttachService.getListBy(Constants.SOURCE_ORG_ISSUE, taOrgIssue.getOrgIssueId().toString());
             taOrgIssue.setAttachList(attachList);
         }

         return ResponseBean.success(taOrgIssue);
     }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taOrgIssue")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                             @ApiParam("问题单ID") @RequestParam(value = "issueId", required = false) String issueId,
                             @ApiParam("点位") @RequestParam(value = "locId", required = false) String locId,
                             @ApiParam("数据来源") @RequestParam(value ="sourceType", required = false) String sourceType,
                             @ApiParam("类型ID") @RequestParam(value = "typeId", required = false) String typeId,
                             @ApiParam("上报人") @RequestParam(value = "userName", required = false) String userName,
                             @ApiParam(value = "创建时间-开始", example = "2023-03-01") @RequestParam(value = "createDateStart", required = false) String createDateStart,
                             @ApiParam(value = "创建时间-结束", example = "2023-03-31") @RequestParam(value = "createDateEnd", required = false) String createDateEnd,
                             @ApiParam(value = "结单时间-开始", example = "2023-03-01") @RequestParam(value = "endDateStart", required = false) String endDateStart,
                             @ApiParam(value = "结单时间-结束", example = "2023-03-31") @RequestParam(value = "endDateEnd", required = false) String endDateEnd,
                             @ApiParam("业务状态") @RequestParam(value ="bizStatus", required = false) String bizStatus) throws Exception {
        SysUser sysUser = currentUser();

        Map<String, Object> params = new HashMap<>();
        params.put("issueId", issueId);
        params.put("typeId", typeId);
        params.put("bizStatus", bizStatus);
        params.put("sourceType", sourceType);
        params.put("locId", locId);
        params.put("orgId", sysUser.getOrgId());
        params.put("userName", userName);
        if (StringUtils.isNotEmpty(createDateStart)) {
            params.put("createDateStart", DateUtils.day2LocalDateime(createDateStart));
            params.put("createDateEnd", DateUtils.day2LocalDateime2(createDateEnd));
        } else {
            params.put("createDateStart", null);
            params.put("createDateEnd", null);
        }

        if (StringUtils.isNotEmpty(endDateStart)) {
            params.put("endDateStart", DateUtils.day2LocalDateime(endDateStart));
            params.put("endDateEnd", DateUtils.day2LocalDateime2(endDateEnd));
        } else {
            params.put("endDateStart", null);
            params.put("endDateEnd", null);
        }

        IPage<TaIssue> pg = new Page<>(pageNum, pageSize);
        IPage<TaIssue> result = taOrgIssueService.getIssuePageBy(pg, params);
        
        return ResponseBean.success(result);
    }


     /**
      * 分页查询
      *
      * @param pageNum 当前页码
      * @param pageSize 每页条数
      * @return 查询结果
      */
     @ApiOperation("分页查询")
     @PostMapping("/taOrgIssue/export")
     public ResponseBean export(@ApiParam("问题单ID") @RequestParam(value = "issueId", required = false) String issueId,
                                @ApiParam("点位") @RequestParam(value = "locId", required = false) String locId,
                                @ApiParam("数据来源") @RequestParam(value ="sourceType", required = false) String sourceType,
                                @ApiParam("业务状态") @RequestParam(value ="bizStatus", required = false) String bizStatus,
                                @ApiParam("类型ID") @RequestParam(value = "typeId", required = false) String typeId,
                                @ApiParam("上报人") @RequestParam(value = "userName", required = false) String userName,
                                @ApiParam(value = "创建时间-开始", example = "2023-03-01") @RequestParam(value = "createDateStart", required = false) String createDateStart,
                                @ApiParam(value = "创建时间-结束", example = "2023-03-31") @RequestParam(value = "createDateEnd", required = false) String createDateEnd,
                                @ApiParam(value = "结单时间-开始", example = "2023-03-01") @RequestParam(value = "endDateStart", required = false) String endDateStart,
                                @ApiParam(value = "结单时间-结束", example = "2023-03-31") @RequestParam(value = "endDateEnd", required = false) String endDateEnd,
                                @ApiParam("是否带图") @RequestParam(value = "withImage", defaultValue = "true") Boolean withImage,
                                HttpServletResponse response) throws Exception {
         SysUser sysUser = currentUser();

         Map<String, Object> params = new HashMap<>();
         params.put("issueId", issueId);
         params.put("typeId", typeId);
         params.put("bizStatus", bizStatus);
         params.put("sourceType", sourceType);
         params.put("locId", locId);
         params.put("orgId", sysUser.getOrgId());
         params.put("userName", userName);
         if (StringUtils.isNotEmpty(createDateStart)) {
             params.put("createDateStart", DateUtils.day2LocalDateime(createDateStart));
             params.put("createDateEnd", DateUtils.day2LocalDateime2(createDateEnd));
         } else {
             params.put("createDateStart", null);
             params.put("createDateEnd", null);
         }

         if (StringUtils.isNotEmpty(endDateStart)) {
             params.put("endDateStart", DateUtils.day2LocalDateime(endDateStart));
             params.put("endDateEnd", DateUtils.day2LocalDateime2(endDateEnd));
         } else {
             params.put("endDateStart", null);
             params.put("endDateEnd", null);
         }

         List<TaIssue> list = taOrgIssueService.getIssueListBy(params);
         List<IssueExport> exportList = exportIssueUtil.getExportList(list, withImage);

         ExcelUtils.flush(response, IssueExport.class, exportList, "问题单列表");
         return null;
     }
    
//    /**
//     * 新增数据
//     *
//     * @param taOrgIssue 实例对象
//     * @return 实例对象
//     */
//    @ApiOperation("新增数据")
//    @PostMapping("/taOrgIssue")
//    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaOrgIssue taOrgIssue) throws Exception {
//        taOrgIssueService.save(taOrgIssue);
//        return ResponseBean.success(taOrgIssue);
//    }
//
    /**
     * 更新数据
     *
     * @param taOrgIssue 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taOrgIssue/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaOrgIssue taOrgIssue,
                            @ApiParam("对象ID") @PathVariable Integer id ) throws Exception {
        taOrgIssue.setOrgIssueId(id);

        if (null == taOrgIssue.getAttachList() || taOrgIssue.getAttachList().size() == 0) {
            return ResponseBean.error("请上传图片");
        }

        taOrgIssueService.updateData(taOrgIssue);
        return ResponseBean.success(taOrgIssue);
    }



    /**
     * 通过主键删除数据
     *
     * @param orgIssueId
     * @return 是否成功
     *
     */

    @ApiOperation(value="通过主键删除数据",httpMethod="DELETE")
    @DeleteMapping("/taOrgIssue/{issueId}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String issueId){
        taOrgIssueService.removeByIssueId(issueId);
        return ResponseBean.success("success成功删除");
    }

//
//    /**
//     * 通过主键删除数据
//     *
//     * @param id 主键
//     * @return 是否成功
//     */
//    @ApiOperation("通过主键删除数据")
//    @DeleteMapping("/taOrgIssue/{id}")
//    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable Integer id){
//        taOrgIssueService.removeLogicById(id);
//        return ResponseBean.success("success");
//    }
}