package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;
import java.util.List;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TaPersonReflect;
import com.example.civilizedcity.service.TaPersonReflectService;

 /**
 * 人员映射表;(ta_person_reflect)表控制层
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "人员映射表对象功能接口")
@RestController
@RequestMapping("/")
public class TaPersonReflectController extends BaseController {
    
    @Autowired
    private TaPersonReflectService taPersonReflectService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param userId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/taPersonReflect/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(taPersonReflectService.getById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/taPersonReflect")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception {
        
        IPage<TaPersonReflect> pg = new Page<>(pageNum, pageSize);
        // QueryWrapper<TaPersonReflect> queryWrapper = new QueryWrapper<>();
        // queryWrapper.orderByDesc("create_date");
        IPage<TaPersonReflect> result = taPersonReflectService.page(pg);
        
        return ResponseBean.success(result);
    }
    
    /** 
     * 新增数据
     *
     * @param taPersonReflect 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/taPersonReflect")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaPersonReflect taPersonReflect) throws Exception {
        taPersonReflectService.save(taPersonReflect);
        return ResponseBean.success(taPersonReflect);
    }
    
    /** 
     * 更新数据
     *
     * @param taPersonReflect 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/taPersonReflect/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaPersonReflect taPersonReflect,
                            @ApiParam("对象ID") @PathVariable String id ) throws Exception {
        taPersonReflectService.updateById(taPersonReflect);
        return ResponseBean.success(taPersonReflect);
    }
    
    /** 
     * 通过主键删除数据
     *
     * @param userId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/taPersonReflect/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id){
        taPersonReflectService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}