package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.ResponseBean;
import java.util.List;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TdIssueType;
import com.example.civilizedcity.service.TdIssueTypeService;

 /**
 * 问题分类;(td_issue_type)表控制层
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "问题分类对象功能接口")
@RestController
@RequestMapping("/")
public class TdIssueTypeController extends BaseController {
    
    @Autowired
    private TdIssueTypeService tdIssueTypeService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param typeId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/tdIssueType/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(tdIssueTypeService.getById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param pageNum 当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/tdIssueType")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                            @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception {
        
        IPage<TdIssueType> pg = new Page<>(pageNum, pageSize);
        // QueryWrapper<TdIssueType> queryWrapper = new QueryWrapper<>();
        // queryWrapper.orderByDesc("create_date");
        IPage<TdIssueType> result = tdIssueTypeService.page(pg);
        
        return ResponseBean.success(result);
    }
    
    /** 
     * 新增数据
     *
     * @param tdIssueType 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/tdIssueType")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TdIssueType tdIssueType) throws Exception {
        tdIssueTypeService.save(tdIssueType);
        return ResponseBean.success(tdIssueType);
    }
    
    /** 
     * 更新数据
     *
     * @param tdIssueType 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/tdIssueType/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TdIssueType tdIssueType,
                            @ApiParam("对象ID") @PathVariable String id ) throws Exception {
        tdIssueType.setTypeId(id);
        tdIssueTypeService.updateById(tdIssueType);
        return ResponseBean.success(tdIssueType);
    }
    
    /** 
     * 通过主键删除数据
     *
     * @param typeId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/tdIssueType/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id){
        tdIssueTypeService.removeLogicById(id);
        return ResponseBean.success("success");
    }




}