package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.util.List;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TdLocType;
import com.example.civilizedcity.service.TdLocTypeService;

/**
 * 点位分类;(td_loc_type)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "点位分类对象功能接口")
@RestController
@RequestMapping("/")
public class TdLocTypeController extends BaseController {

    @Autowired
    private TdLocTypeService tdLocTypeService;

    /**
     * 通过ID查询单条数据
     *
     * @param typeId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/tdLocType/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(tdLocTypeService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/tdLocType")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) throws Exception {

        IPage<TdLocType> pg = new Page<>(pageNum, pageSize);
//        QueryWrapper<TdLocType> queryWrapper = new QueryWrapper<>();
//        queryWrapper.gt("status", Constants.STATUS_DELETE);
//        queryWrapper.orderByDesc("create_date");
//        IPage<TdLocType> result = tdLocTypeService.page(pg, queryWrapper);
        IPage<TdLocType> result = tdLocTypeService.pageAsc(pg);
        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param tdLocType 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/tdLocType")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TdLocType tdLocType) throws Exception {
        tdLocType.setTypeId(null);
        tdLocTypeService.save(tdLocType);
        return ResponseBean.success(tdLocType);
    }

    /**
     * 更新数据
     *
     * @param tdLocType 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/tdLocType/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TdLocType tdLocType,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        tdLocType.setTypeId(id);
        tdLocTypeService.updateById(tdLocType);
        return ResponseBean.success(tdLocType);
    }

    /**
     * 通过主键删除数据
     *
     * @param typeId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/tdLocType/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        tdLocTypeService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}