package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.TdQuAnswer;
import com.example.civilizedcity.service.TaCheckItemAnService;
import com.example.civilizedcity.service.TdQuAnswerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.TdQuestion;
import com.example.civilizedcity.service.TdQuestionService;

import java.util.List;

/**
 * 点位问题;(td_question)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-13
 */
@Api(tags = "点位问题对象功能接口")
@RestController
@RequestMapping("/")
public class TdQuestionController extends BaseController {

    @Autowired
    private TdQuestionService tdQuestionService;

    @Autowired
    private TdQuAnswerService tdQuAnswerService;

    /**
     * 通过ID查询单条数据
     *
     * @param quId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/tdQuestion/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        return ResponseBean.success(tdQuestionService.getById(id));
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/tdQuestion")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("标题") @RequestParam(value = "title", required = false) String title,
                             @ApiParam("问题类型") @RequestParam(value = "quType", required = false) String quType) throws Exception {

        IPage<TdQuestion> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TdQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(title), "title", title);
        queryWrapper.eq(!StringUtils.isEmpty(quType), "qu_type", quType);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.orderByDesc("create_date");
        IPage<TdQuestion> result = tdQuestionService.page(pg, queryWrapper);

        List<TdQuestion> records = result.getRecords();
        if (null != records || records.size() > 0) {
            for (TdQuestion tdQuestion : records) {
                List<TdQuAnswer> answerList = tdQuAnswerService.getListByQuId(tdQuestion.getQuId());
                tdQuestion.setQuAnswerList(answerList);
            }
        }

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param tdQuestion 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/tdQuestion")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TdQuestion tdQuestion) throws Exception {
        String quId = tdQuestion.getQuId();

        if (StringUtils.isEmpty(quId)) {
            // 新增
            tdQuestionService.save(tdQuestion);
        } else {
            tdQuestionService.updateById(tdQuestion);
        }

        tdQuAnswerService.mergeData(tdQuestion.getQuId(), tdQuestion.getQuAnswerList());
        return ResponseBean.success(tdQuestion);
    }

    /**
     * 更新数据
     *
     * @param tdQuestion 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/tdQuestion/{id}")
    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TdQuestion tdQuestion,
                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
        tdQuestionService.updateById(tdQuestion);
        return ResponseBean.success(tdQuestion);
    }

    /**
     * 通过主键删除数据
     *
     * @param quId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/tdQuestion/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        tdQuestionService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}