package com.example.civilizedcity.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.EncryptUtils;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.entity.TaPerson;
import com.example.civilizedcity.mapper.SysUserDutyMapper;
import com.example.civilizedcity.mapper.SysUserMapper;
import com.example.civilizedcity.vo.WxChangePwd;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.entity.SysLogin;
import com.example.civilizedcity.mapper.SysLoginMapper;
import com.example.civilizedcity.service.SysLoginService;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 系统账户表;(sys_login)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-12
 */
@Service
public class SysLoginServiceImpl extends BaseServiceImpl<SysLoginMapper, SysLogin> implements SysLoginService {

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    SysUserDutyMapper sysUserDutyMapper;

    @Override
    public boolean changePassword(SysUser user, WxChangePwd wxChangePwd) {

        String pwd = EncryptUtils.md5(wxChangePwd.getPassword(), user.getUserId());
        baseMapper.updatePassword(user.getUserId(), pwd);

        return false;
    }

    @Override
    public Map<String, Object> loginByPerson(TaPerson taPerson) {
        Map<String, Object> res = new HashMap<>();
        SysUser user = null;

                List<SysUser> sysUserList = sysUserMapper.getListByPhone(taPerson.getPhone());
        if (null == sysUserList || sysUserList.size() < 1) {
            user = new SysUser();
            user.setPhone(taPerson.getPhone());
            user.setIsUser(false);
            user.setOrgId(taPerson.getPersonId()); // 机构ID 使用用户ID , 目前没有使用到
            user.setStatus(Constants.STATUS_NORMAL);
            user.setCreateDate(LocalDateTime.now());
            sysUserMapper.insert(user);

            if (null == sysUserList) {
                sysUserList = new ArrayList<>();
            }

            sysUserList.add(user);
        } else {
            if (sysUserList.size() == 1) {
                user = sysUserList.get(0);
            } else {
                user = sysUserList.stream().filter(x -> x.getIsUser()).findFirst().get();
            }
        }

        // 获取人员身份
        List<String> dutyList = sysUserDutyMapper.getListByUser(user.getUserId());
        user.setDutyList(dutyList);

        // 登录新用户
        StpUtil.login(user.getUserId(), "miniapp");
        res.put("user", user);
        res.put("person", taPerson);
        res.put("token", StpUtil.getTokenValue());

        return res;
    }

    @Override
    public void removeByUser(String userId) {
        QueryWrapper<SysLogin> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        remove(queryWrapper);
    }
}