package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.EncryptUtils;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.mapper.*;
import com.example.civilizedcity.vo.ChangePassword;
import com.example.civilizedcity.vo.LoginParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.SysUserService;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 用户表;(sys_user)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-12
 */
@Service
public class SysUserServiceImpl extends BaseServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    SysLoginMapper sysLoginMapper;

    @Autowired
    SysUserRoleMapper sysUserRoleMapper;

    @Autowired
    SysUserDutyMapper sysUserDutyMapper;

    @Autowired
    TaPersonMapper taPersonMapper;

    String defaultPwd = "123456";

    @Override
    public SysUser login(LoginParam loginParm) throws Exception {
        SysLogin sysLogin = sysLoginMapper.getByAccount(loginParm.getAccount());
        if (null == sysLogin) {
            throw new Exception("用户名或者密码不正确");
        }

        String userId = sysLogin.getUserId();
        if (StringUtils.isEmpty(userId)) {
            throw new Exception("用户名或者密码不正确");
        }

        boolean isRight = checkPassword(loginParm.getPassword(), sysLogin.getPassword(), userId);
        if (!isRight) {
            throw new Exception("用户名或者密码不正确");
        }

        SysUser sysUser = getById(userId);
        if (Constants.STATUS_DELETE == sysUser.getStatus()) {
            throw new Exception("用户账户已删除");
        }

        if (Constants.STATUS_NORMAL != sysUser.getStatus()) {
            throw new Exception("用户已停用");
        }

        return sysUser;
    }

    @Override
    public boolean changePassword(SysUser user, ChangePassword param) throws Exception {
        String passwordEncrypted = EncryptUtils.md5(param.getOriginPassword(), user.getUserId());
        SysLogin sysLogin = sysLoginMapper.getByUserAndPassword(user.getUserId(), passwordEncrypted);

        if (null == sysLogin) {
            throw new Exception("原始密码不正确");
        }

        String newPassword = EncryptUtils.md5(param.getNewPassword(), user.getUserId());
        sysLogin.setPassword(newPassword);

        return sysLoginMapper.updateById(sysLogin) > 0;
    }

    private boolean getBool(Boolean b) {
        return null != b && b;
    }

    /**
     * 校验是否市民
     * 用户的 orgId == 市民的 personId
     * @param sysUser
     * @return
     */
    private boolean isCitizen(SysUser sysUser) {
        if (null == sysUser || StringUtils.isEmpty(sysUser.getOrgId())) {
            return false;
        }

        TaPerson taPerson = taPersonMapper.selectById(sysUser.getOrgId());
        return null != taPerson && taPerson.getStatus() > Constants.STATUS_DELETE;
    }

    @Override
    public boolean mergeData(SysUser sysUser) throws Exception {
        if (StringUtils.isEmpty(sysUser.getName())) {
            throw new Exception("用户名称不能为空");
        }

        LocalDateTime now = LocalDateTime.now();

        boolean newOne = StringUtils.isEmpty(sysUser.getUserId());

        if (!StringUtils.isEmpty(sysUser.getPhone())) {
            SysUser origin = getExistBy("phone", sysUser.getPhone(), false, true);
            if (null != origin) {
                if (!newOne && !origin.getUserId().equals(sysUser.getUserId())) {
                    throw new Exception("手机号码重复");
                }

                newOne = false;
                sysUser.setUserId(origin.getUserId());

                // 如果是市民
                if (isCitizen(origin)) {
                    sysUser.setIsUser(true);
                }
            }
        }

        if (null == sysUser.getStatus()) {
            sysUser.setStatus(Constants.STATUS_NORMAL);
        }

        // 用户表
        if (newOne) {
            sysUser.setCreateDate(now);
            save(sysUser);
        } else {
            updateById(sysUser);
        }

        // 登录账户
        SysLogin sysLogin = sysLoginMapper.getByAccount(sysUser.getPhone());
        if (null == sysLogin) {
            sysLogin = new SysLogin();

            sysLogin.setAccount(sysUser.getPhone());
            sysLogin.setPassword(EncryptUtils.md5(EncryptUtils.md5(defaultPwd), sysUser.getUserId()));
            sysLogin.setUserId(sysUser.getUserId());
            sysLogin.setStatus(Constants.STATUS_NORMAL);
            sysLogin.setCreateDate(now);
            sysLoginMapper.insert(sysLogin);
        }

        // 授权角色
        List<SysRole> rolesList = sysUser.getRolesList();
        if (null == rolesList || rolesList.size() == 0) {
            sysUserRoleMapper.deleteByMap(new HashMap<String, Object>() {{
                put("user_id", sysUser.getUserId());
            }});
        } else {
            List<Integer> idList = new ArrayList<>();
            for (SysRole role : rolesList) {
                SysUserRole item = new SysUserRole();
                item.setUserId(sysUser.getUserId());
                item.setRoleId(role.getRoleId());

                SysUserRole exists = sysUserRoleMapper.getByUserAndRole(sysUser.getUserId(), role.getRoleId());
                if (null != exists) {
                    item.setId(exists.getId());
                } else {
                    sysUserRoleMapper.insert(item);
                }

                idList.add(item.getId());
            }

            // 去除垃圾数据
            sysUserRoleMapper.deleteNotIn(sysUser.getUserId(), idList);
        }

        // 身份表
        mergeDuty(sysUser);

        return true;
    }

    private void mergeDuty(SysUser sysUser) {
        List<String> dutyList = sysUser.getDutyList();
        if (null == dutyList || dutyList.size() == 0) {
            sysUserDutyMapper.deleteByMap(new HashMap<String, Object>() {{
                put("user_id", sysUser.getUserId());
            }});
        } else {
            List<Integer> idList = new ArrayList<>();
            for (String duty : dutyList) {
                SysUserDuty item = new SysUserDuty();
                item.setUserId(sysUser.getUserId());
                item.setDuty(duty);

                SysUserDuty exists = sysUserDutyMapper.getByUserAndDuty(sysUser.getUserId(), duty);
                if (null != exists) {
                    item.setId(exists.getId());
                } else {
                    sysUserDutyMapper.insert(item);
                }

                idList.add(item.getId());
            }

            // 去除垃圾数据
            sysUserDutyMapper.deleteNotIn(sysUser.getUserId(), idList);
        }
    }

    @Override
    public List<String> getRoleList(String userId) {
        return sysUserRoleMapper.getRoleList(userId);
    }

    @Override
    public List<String> getPermissionList(String userId) {
        return sysUserDutyMapper.getListByUser(userId);
    }

    @Override
    public SysUser getByPhone(String phone, String orgId) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", phone);
        queryWrapper.eq("org_id", orgId);
        queryWrapper.last("limit 1");

        return getOne(queryWrapper);
    }

    @Override
    public void createMaUser(SysUser sysUser, SysUser currentUser) {

        LocalDateTime now = LocalDateTime.now();

        sysUser.setIsUser(true);
        sysUser.setOrgId(currentUser.getOrgId());
        sysUser.setIsFromWx(true);

        if (StringUtils.isEmpty(sysUser.getUserId())) {
            sysUser.setCreateUser(currentUser.getUserId());
            sysUser.setCreateDate(now);

            save(sysUser);
        } else {
            updateById(sysUser);
        }

        // 登录账户
        SysLogin sysLogin = sysLoginMapper.getByAccount(sysUser.getPhone());
        if (null == sysLogin) {
            sysLogin = new SysLogin();

            sysLogin.setAccount(sysUser.getPhone());
            sysLogin.setPassword(EncryptUtils.md5(EncryptUtils.md5(defaultPwd), sysUser.getUserId()));
            sysLogin.setUserId(sysUser.getUserId());
            sysLogin.setStatus(Constants.STATUS_NORMAL);
            sysLogin.setCreateDate(now);
            sysLoginMapper.insert(sysLogin);
        }

        // 身份表
        if (null == sysUser.getDutyList() || sysUser.getDutyList().size() == 0) {
            sysUser.setDutyList(new ArrayList<String>(){{
                add(Constants.DUTY_ORG_USER);
            }});
        }
        mergeDuty(sysUser);
    }

    boolean checkPassword(String src, String targ, String salt) {
        String newPass = EncryptUtils.md5(src, salt);
        return targ.equals(newPass);
    }
}