package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.mapper.*;
import com.example.civilizedcity.vo.CheckLocExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.TaCheckItemService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 测评点位;(ta_check_item)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-13
 */
@Service
public class TaCheckItemServiceImpl extends BaseServiceImpl<TaCheckItemMapper, TaCheckItem> implements TaCheckItemService {

    @Autowired
    TaCheckItemQuMapper taCheckItemQuMapper;

    @Autowired
    TaCheckItemMapper taCheckItemMapper;

    @Autowired
    TaCheckItemAnMapper taCheckItemAnMapper;

    @Autowired
    TaCheckAnswerItemMapper taCheckAnswerItemMapper;

    @Autowired
    TaCheckAnswerMapper taCheckAnswerMapper;

    @Autowired
    TaAttachMapper taAttachMapper;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    TaIssueMapper taIssueMapper;

    @Autowired
    TaIssueProcessMapper taIssueProcessMapper;

    @Override
    public IPage<TaCheckItem> getPageBy(IPage<TaCheckItem> pg, String checkId, String itemType) {
        return baseMapper.getPageBy(pg, checkId, itemType);
    }

    @Override
    public void createNewItems(TaCheck taCheck) {
        // 先创建实地测试
        baseMapper.createNewLocItems(taCheck.getCheckId());
        // 再创建调查问卷
        TaCheckItem taCheckItem = new TaCheckItem();
        taCheckItem.setCheckId(taCheck.getCheckId());
        taCheckItem.setItemType(Constants.CHECK_OF_SURVEY);
        taCheckItem.setName("调查问卷");
        save(taCheckItem);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void answer(TaCheckItem taCheckItem, TaCheckAnswer taCheckAnswer, SysUser sysUser) throws Exception {
        List<TaCheckItemQu> quList = taCheckItemQuMapper.getListBy(taCheckItem.getItemId());
        if (null == quList || quList.size() < 1) {
            throw new Exception("校验测评问题列表出错");
        }

        List<TaCheckAnswerItem> answerItemList = taCheckAnswer.getAnswerItemList();
        if (null == answerItemList || answerItemList.size() != quList.size()) {
            throw new Exception("未完成全部测评问题");
        }

        // 答案列表如果没有关联问题ID
        long cnt = answerItemList.stream().filter(f -> null == f.getQuId()).count();
        if (cnt > 0) {
            throw new Exception("验证部分题干信息出错");
        }

//        if (Constants.CHECK_OF_LOC.equals(taCheckItem.getItemType())) {
//            // 如果是实地问卷, 试卷份数不能多于规定份数
//            int count = taCheckAnswerMapper.countByItem(taCheckItem.getItemId());
//            if (count >= taCheckItem.getNum()) {
//                throw new Exception("当前实地问卷已收齐, 无须作答");
//            }
//        }

        // 计算每一题的得分， 以及总分
        // 并同步生成问题单
        double score = calcAnswerItems(sysUser, taCheckItem, taCheckAnswer, quList, answerItemList);
        taCheckAnswer.setScore(score);

        //得到测评得分的值并且插入一条记录
//        double evlScore = taCheckItemMapper.sumEvlScore(taCheckItem.getItemId());
//        taCheckItem.setScore(score);

//        UpdateWrapper<TaCheckItem> updateWrapper = new UpdateWrapper<>();
//        updateWrapper.eq("item_id",taCheckItem.getItemId());
//        taCheckItem.setScore(score);
//
//        taCheckItemMapper.update(taCheckItem, updateWrapper);

        taCheckAnswerMapper.insert(taCheckAnswer);
    }

    @Override
    public boolean updateFullScore(String itemId) {
        return baseMapper.updateFullScore(itemId);
    }

    @Override
    public List<CheckLocExport> getExportByCheck(String checkId, String itemType) {
        return baseMapper.getExportByCheck(checkId, itemType);
    }

    @Override
    public void updateLoc(String checkId){
        baseMapper.updateLoc(checkId);
    }
    @Override
    public void syncLoc(String checkId){
        baseMapper.syncLoc(checkId);
    }

    /**
     * 作答计算分数
     * 试卷总分 = 每一题分数求和
     * 选择题分数 = 选中的选项分数
     * 填空题分数 = 填写的数字 * 对应的分值
     * @param sysUser
     * @param taCheckItem
     * @param taCheckAnswer
     * @param quList
     * @param answerItemList
     * @throws Exception
     */
    private double calcAnswerItems(
            SysUser sysUser,
            TaCheckItem taCheckItem,
            TaCheckAnswer taCheckAnswer,
            List<TaCheckItemQu> quList,
            List<TaCheckAnswerItem> answerItemList
    ) throws Exception{
        LocalDateTime now = LocalDateTime.now();
        //题目分值
        double totalScore = 0.0;
        for(TaCheckAnswerItem item : answerItemList) {
            // 每一题分值
            double score = 0.0;
            // 找到对应的题目
            TaCheckItemQu qu = getQu(quList, item.getQuId());
            // 如果是选择题, 对应的答案
            TaCheckItemAn standAn = null;
            // 是否扣分题
            String computeType = qu.getComputeType();

            if (Constants.QU_OF_FILL.equals(qu.getQuType())) {
                // 如果是填空题, 用户填的应该是数字, 没填就是 0
                Double fillNum = StringUtils.isEmpty(item.getAnswer()) ? 0.0 : Double.parseDouble(item.getAnswer());

                // 填空得分 = 填空的数字 * 每个数字的分数
                score = fillNum * qu.getAnScore();
            } else if (Constants.QU_OF_RADIO.equals(qu.getQuType())) {
                // 如果是选择题, 找到对应的答案
                standAn = taCheckItemAnMapper.getOneBy(qu.getQuId(), item.getAnswerCode());
                score = standAn.getScore();
            }

            // 如果得分超过最大值
            if (score > qu.getMaxScore()) {
                score = qu.getMaxScore();
            }

            // 如果是扣分项
            if ("-".equals(computeType)) {
                score = qu.getMaxScore() + score;
                if (score < 0) {
                    score = 0;
                }
            }

            // 判断是否需要生成问题单
            boolean isIssue = false;
            if (score < qu.getMaxScore()) {
                // 如果当前这个题目, 没有得到满分, 就生成问题单
                isIssue = true;
            }

            // 每份试卷的分数 = 各题目的分数求和
            totalScore += score;

            item.setItemId(StringUtils.uuid());
            item.setAnswerId(taCheckAnswer.getAnswerId());
            item.setScore(score);
            item.setCreateDate(now);
            item.setCreateUser(taCheckAnswer.getCreateUser());
            item.setStatus(Constants.STATUS_NORMAL);
            taCheckAnswerItemMapper.insert(item);

            // 保存附件
            if (null != item.getAttachList()) {
                for (TaAttach taAttach : item.getAttachList()) {
                    taAttach.setAttachId(null);
                    taAttach.setOwnerType(Constants.SOURCE_CHECK_ANSWER);
                    taAttach.setOwnerId(item.getItemId());
                    taAttach.setStatus(Constants.STATUS_NORMAL);
                    taAttach.setCreateUser(taCheckAnswer.getCreateUser());
                    taAttach.setCreateDate(now);
                    taAttachMapper.insert(taAttach);
                }
            }

            // 点位问题，生成问题单
            if (Constants.CHECK_OF_LOC.equals(taCheckItem.getItemType()) && isIssue) {
                createNewIssue(sysUser, taCheckItem, qu, standAn, taCheckAnswer, item);
            }
        }

        return totalScore;
    }

    private void createNewIssue(SysUser sysUser,
                                TaCheckItem taCheckItem,
                                TaCheckItemQu qu,
                                TaCheckItemAn an,
                                TaCheckAnswer answer,
                                TaCheckAnswerItem answerItem) {
        LocalDateTime now = LocalDateTime.now();
        TaIssue taIssue = new TaIssue();
        taIssue.setLocId(taCheckItem.getTypeId());
        taIssue.setLocName(taCheckItem.getName());
        taIssue.setAddr(answer.getAddr());

        // 如果是选择题, 答案 answer 的内容只是代码, 所以需要从 TaCheckItemAn 把对应的文字拿出来
        // 如果是填空题， 那么文字是 答案（数字） + 单位（fillUnit） , 最后结果类似 5 处。
        String answerStr = Constants.QU_OF_RADIO.equals(qu.getQuType())
                ? an.getAnswer()
                : String.format("%s %s", answerItem.getAnswer(), qu.getFillUnit());
        taIssue.setContent(String.format("%s 测评结果 %s", qu.getTitle(), answerStr));
        taIssue.setProcessNode(Constants.PROCESS_START);
        taIssue.setProcessStatus(Constants.APPLY_PASS); // 模拟测评的不需要审核
        taIssue.setProcessNum(0);
        taIssue.setOrgId(null);
        taIssue.setSourceType(Constants.SOURCE_CHECK);
        taIssue.setSourceTarget(answerItem.getItemId()); // 实际作答的ID, 非测评项目表 ID
        taIssue.setStatus(Constants.STATUS_NORMAL);
        taIssue.setCreateUser(sysUser.getUserId());
        taIssue.setUserName(sysUser.getName());
        taIssue.setCreateDate(now);
        taIssueMapper.insert(taIssue);

        // 保存文件
        List<TaAttach> attachList = answerItem.getAttachList();
        if (null != attachList && attachList.size() > 0) {
            for (TaAttach item : attachList) {
                // 生成新对象, 目的是不影响 item;
                TaAttach newItem = new TaAttach();
                newItem.setAttachId(null);
                newItem.setFileType(item.getFileType());
                newItem.setName(item.getName());
                newItem.setUrl(item.getUrl());
                newItem.setOwnerType(Constants.SOURCE_ISSUE);
                newItem.setOwnerId(taIssue.getIssueId().toString());
                newItem.setStatus(Constants.STATUS_NORMAL);
                newItem.setCreateDate(now);
                taAttachMapper.insert(newItem);
            }
        }

        // 流程开始
        TaIssueProcess process = new TaIssueProcess();
        process.setIssueId(taIssue.getIssueId());
        process.setOrgId(sysUser.getOrgId());
        process.setProcessNode(taIssue.getProcessNode());
        process.setProcessStatus(taIssue.getProcessStatus());
        process.setStatus(Constants.STATUS_NORMAL);
        process.setCreateUser(sysUser.getUserId());
        process.setUserName(sysUser.getName());
        process.setCreateDate(LocalDateTime.now());
        taIssueProcessMapper.insert(process);
    }

    private TaCheckItemQu getQu(List<TaCheckItemQu> quList, String quId) {
        return quList.stream().filter(q -> q.getQuId().equals(quId)).findFirst().orElse(null);
    }



}