package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.event.MessagEvent;
import com.example.civilizedcity.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.TaIssueApplyService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 问题处理申请;(ta_issue_apply)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-27
 */
@Service
public class TaIssueApplyServiceImpl extends BaseServiceImpl<TaIssueApplyMapper, TaIssueApply> implements TaIssueApplyService {

    @Autowired
    TaIssueMapper taIssueMapper;

    @Autowired
    TaOrgIssueMapper taOrgIssueMapper;

    @Autowired
    TaIssueProcessMapper taIssueProcessMapper;

    @Autowired
    TaFeedbackMapper taFeedbackMapper;

    @Autowired
    ApplicationEventPublisher applicationEventPublisher;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createNewApply(TaIssueApply taIssueApply, SysUser sysUser) throws Exception {
        // 1. 先生成申请记录
        taIssueApply.setApplyId(null);
        taIssueApply.setCreateUser(sysUser.getUserId());
        taIssueApply.setCreateUserName(sysUser.getName());
        taIssueApply.setOrgId(sysUser.getOrgId());
        taIssueApply.setCreateDate(LocalDateTime.now());
        save(taIssueApply);

        // 2. 更新主记录
        TaIssue taIssue = taIssueMapper.selectById(taIssueApply.getIssueId());
        taIssue.setProcessNode(taIssueApply.getApplyType());
        taIssue.setProcessStatus(Constants.APPLY_READY);
        taIssue.setApplyType(taIssueApply.getApplyType());
        taIssue.setApplyId(taIssueApply.getApplyId());
        taIssueMapper.updateById(taIssue);

        // 3. 更新单位记录
        TaOrgIssue taOrgIssue = taOrgIssueMapper.getByIssueAndOrg(taIssueApply.getIssueId(), taIssueApply.getOrgId());
        taOrgIssue.setProcessNode(taIssue.getProcessNode());
        taOrgIssue.setProcessStatus(taIssue.getProcessStatus());
        taOrgIssueMapper.updateById(taOrgIssue);

        // 4. 生成流程节点
        TaIssueProcess taIssueProcess = new TaIssueProcess();
        taIssueProcess.setIssueId(taIssue.getIssueId());
        taIssueProcess.setOrgId(taIssueApply.getOrgId());
        taIssueProcess.setProcessNode(taIssue.getProcessNode());
        taIssueProcess.setProcessStatus(taIssue.getProcessStatus());
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);

        applicationEventPublisher.publishEvent(new MessagEvent(this, taIssueApply));
    }

    @Override
    public IPage<TaIssueApply> getPageBy(IPage<TaIssueApply> pg, Integer issueId, String applyType, String orgId, String sourceType, Boolean isAll) {

        // 如果 isAll 是 false
        // 那么只查询还未审核的

        return baseMapper.getPageBy(pg, issueId, applyType, orgId, sourceType, isAll);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void verify(TaIssueApply taIssueApply, SysUser sysUser, TaIssueApply origin) throws Exception {
        // 1. 更新审核信息
        origin.setVerifyStatus(taIssueApply.getVerifyStatus());
        origin.setVerifyDesc(taIssueApply.getVerifyDesc());
        origin.setVerifyOrg(sysUser.getOrgId());
        origin.setVerifyUser(sysUser.getUserId());
        origin.setVerifyUserName(sysUser.getName());
        origin.setVerifyDate(LocalDateTime.now());
        updateById(origin); // 至此 origin 里面是最新的信息

        // 2. 生成流程记录
        TaIssueProcess taIssueProcess = new TaIssueProcess();
        taIssueProcess.setIssueId(origin.getIssueId());
        taIssueProcess.setOrgId(origin.getOrgId());
        taIssueProcess.setProcessNode(origin.getApplyType());
        taIssueProcess.setProcessStatus(origin.getVerifyStatus());
        taIssueProcess.setProcessResult(origin.getVerifyDesc());
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);

        // 3. 更新单位问题表
        TaOrgIssue taOrgIssue = taOrgIssueMapper.getByIssueAndOrg(origin.getIssueId(), origin.getOrgId());
        taOrgIssue.setProcessStatus(origin.getVerifyStatus());

        // 4. 更正问题主表
        TaIssue taIssue = taIssueMapper.selectById(taIssueApply.getIssueId());
        switch (origin.getApplyType()) {
            case Constants.PROCESS_APPLY_VERIFY:
                // 问题单审核
                verifyIssue(origin, taIssue, taOrgIssue);
                break;
            case Constants.PROCESS_APPLY_REJECT:
                // 问题单驳回
                verifyReject(origin, taIssue, taOrgIssue);
                break;
            case Constants.PROCESS_APPLY_DELAY:
                // 延期申请
                verifyDelay(origin, taIssue, taOrgIssue);
                break;
            case Constants.PROCESS_APPLY_END:
                // 销单申请
                verifyEnd(origin, taIssue, taOrgIssue);
                break;
            default:
                // 其他
                break;
        }

        taIssue.setUpdateDate(LocalDateTime.now());
        taIssue.setUpdateUser(sysUser.getUserId());

        taIssueMapper.updateById(taIssue);
        taOrgIssueMapper.updateById(taOrgIssue);

        // 再次生成流程记录,
        createProcessByIssue(taIssue, sysUser, origin);

//        // 如果是用户随手拍
//        if (Constants.SOURCE_FEEDBACK.equals(taIssue.getSourceType())) {
//            // 如果是销单了
//            if (taIssue.getProcessNode().equals(Constants.PROCESS_END)) {
//            }
//        }

        applicationEventPublisher.publishEvent(new MessagEvent(this, origin));
    }

    private void createProcessByIssue(TaIssue taIssue, SysUser sysUser, TaIssueApply taIssueApply) {
        TaIssueProcess taIssueProcess = new TaIssueProcess();
        taIssueProcess.setIssueId(taIssue.getIssueId());
        taIssueProcess.setOrgId(taIssue.getOrgId());
        taIssueProcess.setProcessNode(taIssue.getProcessNode());
        taIssueProcess.setProcessStatus(taIssue.getProcessStatus());
        taIssueProcess.setProcessResult(taIssueApply.getVerifyDesc());
//        taIssueProcess.setProcessResult(null);
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);
    }

    // 问题单审核
    public void verifyIssue(TaIssueApply taIssueApply, TaIssue taIssue, TaOrgIssue taOrgIssue) throws Exception {
        taIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taIssue.setProcessStatus(taIssueApply.getVerifyStatus());

        taOrgIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taOrgIssue.setProcessStatus(taIssue.getProcessStatus());
    }

    // 驳回审核
    public void verifyReject(TaIssueApply taIssueApply, TaIssue taIssue, TaOrgIssue taOrgIssue) throws Exception {
        if (Constants.APPLY_REJECT.equals(taIssueApply.getVerifyStatus())) {
            // 如果审核不通过
            taIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
            taIssue.setProcessStatus(Constants.APPLY_REJECT);
            taIssue.setProcessNum(taIssue.getProcessNum() + 1); // 交办次数 + 1

            // 单位相当于重新开始
            taOrgIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
            taOrgIssue.setProcessStatus(taIssue.getProcessStatus()); // 但是状态是审核不通过
        } else {
            taIssue.setProcessNode(Constants.PROCESS_START); // 问题单重新开始
            taIssue.setProcessStatus(Constants.APPLY_PASS);
//            taIssue.setProcessNum(0); // 交办次数清零
            taIssue.setOrgId(null); // 没有交办单位
            taIssue.setExpireDate(null); // 截止时间也没了
            taIssue.setApplyId(null);
            taIssue.setApplyType(null);

            // 单位相当于结单了
            taOrgIssue.setProcessNode(Constants.PROCESS_END);
            taOrgIssue.setProcessStatus(Constants.APPLY_PASS);
        }
    }

    // 延期审核
    public void verifyDelay(TaIssueApply taIssueApply, TaIssue taIssue, TaOrgIssue taOrgIssue) throws Exception {
        if (Constants.APPLY_PASS.equals(taIssueApply.getVerifyStatus())) {
            taIssue.setExpireDate(taIssueApply.getContent()); // 更新延期日期
        }

        taIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taIssue.setProcessStatus(taIssueApply.getVerifyStatus());

        taOrgIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taOrgIssue.setProcessStatus(taIssueApply.getVerifyStatus());
    }

    // 销单审核
    public void verifyEnd(TaIssueApply taIssueApply, TaIssue taIssue, TaOrgIssue taOrgIssue) throws Exception {
        if (Constants.APPLY_REJECT.equals(taIssueApply.getVerifyStatus())) {
            // 如果审核不通过
            taIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
            taIssue.setProcessStatus(Constants.APPLY_REJECT);
            taIssue.setProcessNum(taIssue.getProcessNum() + 1); // 交办次数 + 1

            taOrgIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
            taOrgIssue.setProcessStatus(taIssue.getProcessStatus());
        } else {
            taIssue.setProcessNode(Constants.PROCESS_END);
            taIssue.setProcessStatus(Constants.APPLY_PASS);
            taIssue.setEndDate(LocalDateTime.now());
            taIssue.setApplyId(null);
            taIssue.setApplyType(null);

            taOrgIssue.setProcessNode(Constants.PROCESS_END);
            taOrgIssue.setProcessStatus(Constants.APPLY_PASS);
        }
    }


}