package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.event.MessagEvent;
import com.example.civilizedcity.mapper.*;
import com.example.civilizedcity.vo.StatVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.TaIssueService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 问题表;(ta_issue)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-12
 */
@Service
public class TaIssueServiceImpl extends BaseServiceImpl<TaIssueMapper, TaIssue> implements TaIssueService {

    @Autowired
    TaOrgIssueMapper taOrgIssueMapper;

    @Autowired
    TaFeedbackMapper taFeedbackMapper;

    @Autowired
    TaIssueProcessMapper taIssueProcessMapper;

    @Autowired
    TaAttachMapper taAttachMapper;

    @Autowired
    ApplicationEventPublisher applicationEventPublisher;

    @Override
    public IPage<TaIssue> getPageBy(IPage<TaIssue> pg, String orgId) {
        if (null == orgId) {
            return baseMapper.getAllBy(pg);
        }


        return baseMapper.getPageBy(pg, orgId);
    }

    @Override
    public void updateValidateStatus(Integer issueId, int validateStatus) {
        baseMapper.updateValidateStatus(issueId, validateStatus);
    }

    @Override
    public void updateApply(Integer issueId, String applyType, Integer applyId) {
        baseMapper.updateApply(issueId, applyType, applyId);
    }

    @Override
    public List<StatVo> statMaIndex(SysUser user, TaPerson person, String duty) {
        List<StatVo> result = new ArrayList<>();

        // 督察员
        if (Constants.DUTY_INSPECTOR.equals(duty)) {
            Map<String, Object> res = baseMapper.statMaIndex(user.getUserId());
            result.add(new StatVo().setName("已上报").setValue(getStatValueBy(res.get("published_num"), 0)));
            result.add(new StatVo().setName("已交办").setValue(getStatValueBy(res.get("assigned_num"), 0)));
            result.add(new StatVo().setName("未交办").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
        }

        // 管理员
        else if (Constants.DUTY_MANAGER.equals(duty)) {
            Map<String, Object> res = baseMapper.statMaIndex(null);
            result.add(new StatVo().setName("待交办").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
            result.add(new StatVo().setName("已交办").setValue(getStatValueBy(res.get("assigned_num"), 0)));
            result.add(new StatVo().setName("已办结").setValue(getStatValueBy(res.get("end_num"), 0)));
        }

        // 单位人员 或者 单位管理员
        else if (Constants.DUTY_ORG_USER.equals(duty) || Constants.DUTY_ORG_MANAGER.equals(duty)) {
            Map<String, Object> res = taOrgIssueMapper.statMaIndex(user.getOrgId());
            result.add(new StatVo().setName("处理中").setValue(getStatValueBy(res.get("doing_num"), 0)));
            result.add(new StatVo().setName("已办结").setValue(getStatValueBy(res.get("end_num"), 0)));
            result.add(new StatVo().setName("已逾期").setValue(getStatValueBy(res.get("delay_num"), 0)));
        }

        // 查询人员
        else if (Constants.DUTY_QUERY_PERSON.equals(duty)) {
            Map<String, Object> res = baseMapper.statMaIndex(null);
            result.add(new StatVo().setName("未处理").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
            result.add(new StatVo().setName("处理中").setValue(getStatValueBy(res.get("assigned_num"), 0)));
            result.add(new StatVo().setName("已办结").setValue(getStatValueBy(res.get("end_num"), 0)));
        }

        // 市民
        else {
            Map<String, Object> res = taFeedbackMapper.statMaIndex(person.getPersonId());
            if (null == res) {
                res = new HashMap<>();
            }
            result.add(new StatVo().setName("未处理").setValue(getStatValueBy(res.get("unassigned_num"), 0)));
            result.add(new StatVo().setName("已处理").setValue(getStatValueBy(res.get("assigned_num"), 0) + getStatValueBy(res.get("end_num"), 0)));
            result.add(new StatVo().setName("已打回").setValue(getStatValueBy(res.get("reject_num"), 0)));
        }

        return result;
    }

    @Override
    public void updateEnd(SysUser sysUser, String orgId, Integer issueId) {
        // 问题单完成
        baseMapper.updateProcess(issueId, sysUser.getUserId(), Constants.PROCESS_END, Constants.PROCESS_END);

        // 机构问题单完成
        taOrgIssueMapper.updateProcess(issueId, orgId, Constants.PROCESS_END);

        // 流程完成
        TaIssueProcess taIssueProcess = new TaIssueProcess();
        taIssueProcess.setIssueId(issueId);
        taIssueProcess.setOrgId(orgId);
        taIssueProcess.setProcessNode(Constants.PROCESS_END);
        taIssueProcess.setProcessStatus(Constants.PROCESS_END);
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createNew(TaIssue taIssue, SysUser user) throws Exception {
        taIssue.setIssueId(null);
        taIssue.setOrgId(null); // 暂时没有责任单位
        taIssue.setProcessNode(Constants.PROCESS_START);
        taIssue.setProcessStatus(Constants.APPLY_READY); // 督察员上报需要审核
        taIssue.setProcessNum(0);   // 交办次数
        taIssue.setSourceType(Constants.SOURCE_INSPECTOR);
        taIssue.setStatus(Constants.STATUS_NORMAL);
        taIssue.setCreateUser(user.getUserId());
        taIssue.setUserName(user.getName());
        taIssue.setCreateDate(LocalDateTime.now());
        save(taIssue);

        // 保存文件
        List<TaAttach> attachList = taIssue.getAttachList();
        if (null != attachList && attachList.size() > 0) {
            for (TaAttach item : attachList) {
                item.setAttachId(null);
                item.setOwnerType(Constants.SOURCE_ISSUE);
                item.setOwnerId(taIssue.getIssueId().toString());
                item.setStatus(Constants.STATUS_NORMAL);
                item.setCreateDate(LocalDateTime.now());
                taAttachMapper.insert(item);
            }
        }

        // 流程开始
        TaIssueProcess process = new TaIssueProcess();
        process.setIssueId(taIssue.getIssueId());
        process.setOrgId(user.getOrgId());
        process.setProcessNode(taIssue.getProcessNode());
        process.setProcessStatus(taIssue.getProcessStatus());
        process.setStatus(Constants.STATUS_NORMAL);
        process.setCreateUser(user.getUserId());
        process.setUserName(user.getName());
        process.setCreateDate(LocalDateTime.now());
        taIssueProcessMapper.insert(process);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateIssue(TaIssue taIssue, TaIssue origin) throws Exception {
        // 更新的时候只允许非业务流程字段
        origin.setTitle(taIssue.getTitle());
        origin.setTypeId(taIssue.getTypeId());
        origin.setTypeName(taIssue.getTypeName());
        origin.setLocId(taIssue.getLocId());
        origin.setLocName(taIssue.getLocName());
        origin.setLocation(taIssue.getLocation());
        origin.setAddr(taIssue.getAddr());
        origin.setContent(taIssue.getContent());
        updateById(origin);

        // 保存文件
        List<TaAttach> attachList = taIssue.getAttachList();
        List<Integer> idList = new ArrayList<>();
        for (TaAttach item : attachList) {
            item.setOwnerType(Constants.SOURCE_ISSUE);
            item.setOwnerId(taIssue.getIssueId().toString());
            item.setStatus(Constants.STATUS_NORMAL);
            item.setCreateDate(LocalDateTime.now());

            if (null == item.getAttachId()) {
                taAttachMapper.insert(item);
            } else {
                taAttachMapper.updateById(item);
            }
            idList.add(item.getAttachId());
        }

        // 删除部分文件
        taAttachMapper.deleteNotIn(idList, Constants.SOURCE_ISSUE, origin.getIssueId().toString());
        origin.setAttachList(attachList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void reject(TaIssue taIssue, TaIssueProcess taIssueProcess, SysUser user) throws Exception {
        // 问题单
        if (Constants.SOURCE_FEEDBACK.equals(taIssue.getSourceType())) {
            // 如果问题单是市民提的
            taIssue.setProcessNode(Constants.PROCESS_END);
            TaFeedback taFeedback = taFeedbackMapper.selectById(taIssue.getSourceTarget());
            if (null == taFeedback) {
                throw new Exception("未找到市民随手拍记录");
            }

            taFeedback.setVerifyStatus(Constants.APPLY_REJECT);
            taFeedback.setVerifyDesc(taIssueProcess.getProcessResult());
            taFeedback.setVerifyDate(LocalDateTime.now());
            taFeedbackMapper.updateById(taFeedback);
        } else {
            taIssue.setProcessNode(Constants.PROCESS_START);
        }
        taIssue.setProcessStatus(Constants.APPLY_REJECT); // 驳回
        taIssue.setUpdateUser(user.getUserId());
        taIssue.setUpdateDate(LocalDateTime.now());
        baseMapper.updateById(taIssue);

        // 流程记录
        taIssueProcess.setProcessId(null);
        taIssueProcess.setProcessNode(taIssue.getProcessNode());
        taIssueProcess.setProcessStatus(taIssue.getProcessStatus());
        taIssueProcess.setCreateUser(user.getUserId());
        taIssueProcess.setUserName(user.getName());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcessMapper.insert(taIssueProcess);

        // 发送消息
        applicationEventPublisher.publishEvent(new MessagEvent(this, taIssue));
    }

    @Override
    public TaIssue copyIssue(TaIssue origin, SysUser sysUser) {

        TaIssue taIssue = new TaIssue();
        taIssue.setCopyFrom(origin.getIssueId().toString());
        taIssue.setTitle(origin.getTitle());
        taIssue.setTypeId(origin.getTypeId());
        taIssue.setTypeName(origin.getTypeName());
        taIssue.setLocId(origin.getLocId());
        taIssue.setLocName(origin.getLocName());
        taIssue.setLocation(origin.getLocation());
        taIssue.setAddr(origin.getAddr());
        taIssue.setContent(origin.getContent());
        taIssue.setProcessNode(Constants.PROCESS_START);
        taIssue.setProcessNum(0);
        taIssue.setSourceType(Constants.SOURCE_INSPECTOR);
        taIssue.setStatus(Constants.STATUS_NORMAL);
        taIssue.setCreateUser(origin.getCreateUser());
        taIssue.setCreateDate(LocalDateTime.now());
        taIssue.setUserName(origin.getUserName());
        taIssue.setUserDuty(origin.getUserDuty());

        save(taIssue);

        // 流程开始
        TaIssueProcess process = new TaIssueProcess();
        process.setIssueId(taIssue.getIssueId());
        process.setOrgId(sysUser.getOrgId());
        process.setProcessNode(taIssue.getProcessNode());
        process.setProcessStatus(taIssue.getProcessStatus());
        process.setStatus(Constants.STATUS_NORMAL);
        process.setCreateUser(sysUser.getUserId());
        process.setUserName(sysUser.getName());
        process.setCreateDate(LocalDateTime.now());
        taIssueProcessMapper.insert(process);

        // copy 图片
        taAttachMapper.copy(Constants.SOURCE_ISSUE, origin.getIssueId().toString(), taIssue.getIssueId().toString());

        return taIssue;
    }

    Integer getStatValueBy(Object v, Integer def) {
        if (null == v) return def;
        return Integer.valueOf(v.toString());
    }
}