package com.example.civilizedcity.service.impl;

import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.TaIssue;
import com.example.civilizedcity.entity.TaIssueApply;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.entity.TaMessage;
import com.example.civilizedcity.mapper.TaMessageMapper;
import com.example.civilizedcity.service.TaMessageService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 消息通知;(ta_message)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2023-1-2
 */
@Service
public class TaMessageServiceImpl extends BaseServiceImpl<TaMessageMapper, TaMessage> implements TaMessageService {

    /**
     * 平台流程
     * @param taIssue
     */
    @Async
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sendMessage(TaIssue taIssue) {
        TaMessage taMessage = new TaMessage();

        // 问题单驳回
        if (Constants.APPLY_REJECT.equals(taIssue.getProcessStatus())) {
            taMessage.setMsgType("issue-reject");
            taMessage.setTitle("问题单被驳回");
            taMessage.setTargetType(Constants.SOURCE_ISSUE);
            taMessage.setTargetId(taIssue.getIssueId().toString());
            taMessage.setStatus(Constants.STATUS_NORMAL);
            taMessage.setCreateDate(LocalDateTime.now());

            if (Constants.SOURCE_FEEDBACK.equals(taIssue.getSourceType())) {
                // 如果是市民上报
                // 不发消息
            }

            if (Constants.SOURCE_INSPECTOR.equals(taIssue.getSourceType())) {
                // 如果是督察员上报
                taMessage.setOrgId(Constants.ROOT_ORG);
                taMessage.setReceiveUser(taIssue.getCreateUser());
            }

            save(taMessage);
        }
    }

    /**
     * 单位流程
     * @param taIssueApply
     */
    @Async
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sendMessage(TaIssueApply taIssueApply) {
        boolean isApply = null == taIssueApply.getVerifyDate();
        TaMessage taMessage = new TaMessage();
        taMessage.setTargetType(Constants.SOURCE_APPLY);
        taMessage.setTargetId(taIssueApply.getApplyId().toString());
        taMessage.setStatus(Constants.STATUS_NORMAL);
        taMessage.setCreateDate(LocalDateTime.now());

        if (isApply) {
            taMessage.setMsgType("apply-" + taIssueApply.getApplyType());

            switch (taIssueApply.getApplyType()) {
                case Constants.PROCESS_APPLY_DELAY:
                    taMessage.setTitle("延期申请");
                    taMessage.setContent("延期至: " + taIssueApply.getContent());
                    taMessage.setReceiveUser(null);
                    taMessage.setOrgId(Constants.ROOT_ORG);
                    break;
                case Constants.PROCESS_APPLY_REJECT:
                    taMessage.setTitle("驳回申请");
                    taMessage.setContent(null);
                    taMessage.setReceiveUser(null);
                    taMessage.setOrgId(Constants.ROOT_ORG);
                    break;
                case Constants.PROCESS_APPLY_VERIFY:
                    taMessage.setTitle("审核申请");
                    taMessage.setContent(null);
                    taMessage.setReceiveUser(null);
                    taMessage.setOrgId(taIssueApply.getOrgId());
                    break;
                case Constants.PROCESS_APPLY_END:
                    taMessage.setTitle("消单申请");
                    taMessage.setContent(null);
                    taMessage.setReceiveUser(null);
                    taMessage.setOrgId(Constants.ROOT_ORG);
                    break;
                default:
                    taMessage.setTitle("未知申请");
                    taMessage.setReceiveUser(null);
                    taMessage.setOrgId(null);
                    break;
            }
        } else {
            String content = "ID: "+ taIssueApply.getIssueId()  +" ， 审核结果: " + (Constants.APPLY_PASS.equals(taIssueApply.getVerifyStatus()) ? "通过" : "不通过");
            content += String.format("。 意见： %s", StringUtils.ifNull(taIssueApply.getVerifyDesc(), "无"));

            taMessage.setMsgType("verify-" + taIssueApply.getApplyType());
            taMessage.setContent(content);
            taMessage.setReceiveUser(null);
            taMessage.setOrgId(taIssueApply.getOrgId());

            switch (taIssueApply.getApplyType()) {
                case Constants.PROCESS_APPLY_DELAY:
                    taMessage.setTitle("延期审核");
                    break;
                case Constants.PROCESS_APPLY_REJECT:
                    taMessage.setTitle("驳回审核");
                    break;
                case Constants.PROCESS_APPLY_VERIFY:
                    taMessage.setTitle("问题单审核");
                    break;
                case Constants.PROCESS_APPLY_END:
                    taMessage.setTitle("办结审核");
                    break;
                default:
                    taMessage.setTitle("未知审核");
                    taMessage.setReceiveUser(null);
                    taMessage.setOrgId(null);
                    break;
            }
        }

        save(taMessage);
    }
}