package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.TaOrgIssueService;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 单位问题单;(ta_org_issue)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-28
 */
@Service
public class TaOrgIssueServiceImpl extends BaseServiceImpl<TaOrgIssueMapper, TaOrgIssue> implements TaOrgIssueService {

    @Autowired
    TaIssueMapper taIssueMapper;

    @Autowired
    TaIssueProcessMapper taIssueProcessMapper;

    @Autowired
    TaFeedbackMapper taFeedbackMapper;

    @Autowired
    TaAttachMapper taAttachMapper;


    private void transParams(Map<String, Object> params) {
        params.put("processNode", null);
        params.put("isExpire", false);
        params.put("expiredEnd", false);

        String bizStatus = (String) params.get("bizStatus");
        if (!StringUtils.isEmpty(bizStatus)) {
            if ("expire".equals(bizStatus) || "expired".equals(bizStatus)) {
                params.put("isExpire", true);
            } else if ("expiredEnd".equals(bizStatus)) {
                params.put("expiredEnd", true);
            } else {
                params.put("processNode", bizStatus);
            }
        }
    }

    @Override
    public IPage<TaIssue> getIssuePageBy(IPage<TaIssue> pg, Map<String, Object> params) {
        transParams(params);
        return baseMapper.getIssuePageBy(pg, params);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createNewIssue(TaIssue taIssue, TaIssueProcess taIssueProcess, SysUser sysUser) throws Exception {
        if (Constants.SOURCE_FEEDBACK.equals(taIssue.getSourceType())) {
            // 如果是市民提的问题单
            TaFeedback taFeedback = taFeedbackMapper.selectById(taIssue.getSourceTarget());
            if (null == taFeedback) {
                throw new Exception("未找到市民随手拍记录");
            }

            taFeedback.setVerifyDate(LocalDateTime.now());
            taFeedback.setVerifyStatus(Constants.APPLY_PASS);
            taFeedbackMapper.updateById(taFeedback);
        }

        // 1. 先修改主记录信息
        taIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taIssue.setProcessStatus(Constants.APPLY_PASS);
        taIssue.setOrgId(taIssueProcess.getNextOrg());
        taIssue.setExpireDate(taIssueProcess.getExpireDate());
        taIssue.setProcessNum(taIssue.getProcessNum() + 1); // 交办次数
        taIssueMapper.updateById(taIssue);

        // 2. 新增机构问题单
        TaOrgIssue taOrgIssue = new TaOrgIssue();
        taOrgIssue.setIssueId(taIssue.getIssueId());
        taOrgIssue.setOrgId(taIssue.getOrgId());
        taOrgIssue.setProcessNode(Constants.PROCESS_ASSIGNED);
        taOrgIssue.setProcessStatus(null);
        taOrgIssue.setStatus(Constants.STATUS_NORMAL);
        taOrgIssue.setCreateDate(LocalDateTime.now());
        taOrgIssue.setCreateUser(sysUser.getUserId());
        save(taOrgIssue);

        // 3. 保存流程节点记录
        taIssueProcess.setProcessId(null);
        taIssueProcess.setProcessNode(Constants.PROCESS_ASSIGNED);
        taIssueProcess.setProcessStatus(Constants.APPLY_PASS);
        taIssueProcess.setCreateUser(sysUser.getUserId());
        taIssueProcess.setCreateDate(LocalDateTime.now());
        taIssueProcess.setStatus(Constants.STATUS_NORMAL);
        taIssueProcessMapper.insert(taIssueProcess);
    }

    @Override
    public TaOrgIssue getByIssueAndOrg(Integer issueId, String orgId) {
        return baseMapper.getByIssueAndOrg(issueId, orgId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateData(TaOrgIssue taOrgIssue) throws Exception {
        updateById(taOrgIssue);

        // 保存文件
        List<TaAttach> attachList = taOrgIssue.getAttachList();
        List<Integer> idList = new ArrayList<>();
        for (TaAttach item : attachList) {
            item.setOwnerType(Constants.SOURCE_ORG_ISSUE);
            item.setOwnerId(taOrgIssue.getOrgIssueId().toString());
            item.setStatus(Constants.STATUS_NORMAL);
            item.setCreateDate(LocalDateTime.now());

            if (null == item.getAttachId()) {
                taAttachMapper.insert(item);
            } else {
                taAttachMapper.updateById(item);
            }
            idList.add(item.getAttachId());
        }

        // 删除部分文件
        taAttachMapper.deleteNotIn(idList, Constants.SOURCE_ORG_ISSUE, taOrgIssue.getOrgIssueId().toString());
        taOrgIssue.setAttachList(attachList);
    }

    @Override
    public List<TaIssue> getIssueListBy(Map<String, Object> params) {
        transParams(params);
        return baseMapper.getIssueListBy(params);
    }

    @Override
    public void removeByIssueId(String issueId){
        baseMapper.removeByIssueId(issueId);
        taIssueMapper.removeByIssueId(issueId);
    }

}